      program p2drcn
c    This is a main program for the solution to the Robin boundary
c     value problem for the Poisson equation in 2-D
c -[d(KXX du/dx)/dx+d(KXY du/dy)/dx+d(KXY du/dx)/dy+d(KYY du/dy)/dy=f
c nx*(KXX du/dx + KXY du/dy) +ny*(KXY du/dx + KYY du/dy) +
c + alpha*u = gamma for (x,y) on the boundary.
c where nx,ny - components of outward unit normal to the boundary.
c=================================================================
c     Discretization - SCALAR functions in CELLS (HC),
c                    - VECTOR functions in NODES (HN)
c=================================================================
      implicit real*8 (a-h,o-z)
c=================================================================
c     eps - constant in the condition of finishing the iteration in 
c     the Gauss-Siedel iteration procedure.  The termination condition is
c
c             (S+1)     (S)
c     max(abs( U(i,j) - U(i,j))) < eps
c     i,j
c 
c     where S is the iteration number.
c

      common/eps/ eps
c=====================================================================

c     Set of coordinates of grid points
c     call subroutine mkgrid

      call mkgrid

c     Set boundary conditions
c     call subroutine bcon

      call bcon


c     Computation of coefficient K in cells
c     call subroutine coeff

      call coeff
      

c     Computation of the right-hand side of the differential equation in nodes
c     call subroutine right

      call right

c     Computation of coefficients and the right-hand side of the
c     finite-difference equations
c     call subroutine CFDS
      call CFDS

c     Set of constants for the termination of the iteration process
      eps=1.d-8  

c     Solution for the finite-difference equations by the GAUSS-SIEDEL method
c     call subroutine siedel
      call siedel

c     Computing the error
c     call subroutine error

      call error

c-------------------------------------------------------------------------
c     Test output
c-------------------------------------------------------------------------
      call output
      stop
      end

c*************************************************************************

      subroutine CFDS
c     This is subroutine CFDS for the computation of coefficents
c     and the right-hand side of the finite-difference scheme for the
c     Robin value problem for the Poisson equation in 2-D
c -[d(KXX du/dx)/dx+d(KXY du/dy)/dx+d(KXY du/dx)/dy+d(KYY du/dy)/dy=f
c nx*(KXX du/dx + KXY du/dy) +ny*(KXY du/dx + KYY du/dy) +
c + alpha*u = gamma for (x,y) on the boundary.
c where nx,ny - components of outward unit normal to the boundary.
c=================================================================
c     Discretization - SCALAR functions in CELLS (HC),
c                    - VECTOR functions in NODES (HN)
c=================================================================
      implicit real*8(a-h,o-z)

c     M,N - number of nodes in the grid 
      common/MN/ M,N

c     X,Y - arrays (0 to M+1 and 0 to N+1), coordinates of grid nodes
c     X and Y for i=0,M+1; j=0,N+1 are fictitious, and are used
c     for making the computational formulas and code shorter.
      common/xy/ x(0:201,0:201),y(0:201,0:201)

c     ZK.. - arrays of (1 to M and 1 to N), value for the matrix of
c     coefficient K in nodes
      common/k/ zkxx(200,200), zkxy(200,200),zkyy(200,200)

c      ALPHAS, ALPHAE, ALPHAN, ALPHAW - arrays which correspond
c     to function alpha(x,y) in the left-hand side of the boundary
c     conditions: - GIVEN in the middle of the boundary segments
c     ALPHAS, ALPHAN - south and north boundary - arrays (1 to M-1 )
c     ALPHAW, ALPHAE - west and east boundary - arrays ( 1 to N-1  )
      common/alpha/ alphas(200), alphae(200),
     * alphan(200), alphaw(200)

c     GAMMAS, GAMMAE, GAMMAN, GAMMAW - arrays which correspond
c     to function gamma(x,y) in the right-hand side of the
c     boundary conditions: - GIVEN in the middle of the boundary
c     GAMMAS, GAMMAN - south and north boundary - arrays (1 to M-1 )
c     GAMMAW, GAMMAE - west and east boundary - arrays ( 1 to N-1 )
      common/gamma/ gammas(200), gammae(200),
     * gamman(200), gammaw(200)


c     F - array (1 to M-1, and 1 to N-1), values for the right-hand
c     side of the differential equation in cells
      common/f/ f(200,200)


c     =================================================================
c     CC,CE,CNE,CN,CNW,CW,CSW,CS,CSE, PHI - arrays of coefficients
c                                           and the right-hand side
c     ( 0 to M and 0 to N ) in finite-difference equations
c
c     C(i,j)*U(i,j)+
c
c     E(i,j)*U(i+1,j) + NE(i,j)*U(i+1,j+1) + N(i,j)*U(i,j+1) +
c
c     NW(i,j)*U(i-1,j+1) + W(i,j)*U(i-1,j) +SW(i,j)*U(i-1,j-1) +
c
c     S(i,j)*U(i,j-1) + SE(i,j)*U(i+1,j-1) = PHI(i,j)
c


c------------------------GENERAL  STENCIL ----------------------------
c
c
c
c              (i-1,j+2)     (i,j+2)         (i+1,j+2)         (i+2,j+2)
c             *-------------*---------------*-----------------*
c             !             !               !                 !
c             !             !               !                 !
c             !             !               !                 !                 
c             !    NW(i,j)  !   N(i,j)      !   NE(i,j)       !                 
c             !             !               !                 !
c             !             !               !                 ! 
c             !(i-1,j+1)    !(i,j+1)        !(i+1,j+1)        !(i+2,j+1)
c             *-------------*---------------*-----------------*
c             !             !               !                 !
c             !             !               !                 !
c             !             !               !                 !
c             !    W(i,j)   !   C(i,j)      !   E(i,j)        !
c             !             !               !                 !
c             !             !               !                 !
c             !(i-1,j)      !(i,j)          !(i+1,j)          !(i+2,j)
c             *-------------*---------------*-----------------*
c             !             !               !                 !
c             !             !               !                 !
c             !             !               !                 !
c             !    SW(i,j)  !   S(i,j)      !   SE(i,j)       !                 
c             !             !               !                 !
c             !             !               !                 !
c             !(i-1,j-1)    !(i,j-1)        !(i+1,j-1)        !(i+2,j-1)
c             *-------------*---------------*-----------------*
c
c
c
c
c
c----------------------  STENCIL --------------------------------

c
c
      common/coefds/
     *      cc(0:200,0:200),ce(0:200,0:200),
     *      cne(0:200,0:200),cn(0:200,0:200),
     *      cnw(0:200,0:200),cw(0:200,0:200),
     *      csw(0:200,0:200),cs(0:200,0:200),
     *      cse(0:200,0:200),phi(0:200,0:200)
c=================================================================
c     VC, VN - arrays for the volume of cells and nodes
      dimension VC(200,200), VN(200,200)

c     ZLS, ZLE, ZLN, ZLW - arrays for the length of boundary sides
c                          on the boundaries
c     ZLS - south, ZLE - east, ZLN - north, ZLW - west.
      dimension zls(200), zle(200), zln(200), zlw(200)


c     Computing VC and VN

      do  i=1,M-1
      do  j=1,N-1

      VC(i,j) =0.5*((x(i,j)-x(i+1,j+1))*(y(i+1,j)-y(i,j+1)) -
     *              (x(i+1,j)-x(i,j+1))*(y(i,j)-y(i+1,j+1)))

      end do
      end do

      do  i=1,M
      do  j=1,N
c     Two different choices for VN(i,j)
c     First - average of volumes of adjacent cells
c     VN(i,j)=0.25*(VC(i,j) + VC(i-1,j) + VC(i-1,j-1) + VC(i,j-1))
c     Second "DIAMOND" volume
      VN(i,j) =0.25*((x(i-1,j)-x(i+1,j))*(y(i,j-1)-y(i,j+1)) -
     *              (x(i,j-1)-x(i,j+1))*(y(i-1,j)-y(i+1,j)))
c
      end do
      end do


c    Computing the length of boundary sides zls,zle,zln,zlw

c    South and North boundary 
      do i=1,M-1
      zls(i) = sqrt((x(i+1,1)-x(i,1))**2 +
     *              (y(i+1,1)-y(i,1))**2)

      zln(i) = sqrt((x(i+1,N)-x(i,N))**2 +
     *              (y(i+1,N)-y(i,N))**2)
      end do

c     East and West boundary 
      do j=1,N-1
      zle(j) = sqrt((x(M,j+1)-x(M,j))**2 +
     *              (y(M,j+1)-y(M,j))**2)

      zlw(j) = sqrt((x(1,j+1)-x(1,j))**2 +
     *              (y(1,j+1)-y(1,j))**2)
      end do


c     computing the coefficients of the finite-difference scheme
c     and the right-hand side
c     ========================================================

c     Approximation of equation div K grad u = f in cells
c     i=1,M-1; j=1,N-1

      do  i=1,M-1
      do  j=1,N-1
c
c     COEFFICIENT E(I,J), East, CELL (I+1,J)
c
c    group related to the approximation of term d(zkxx du/dx)/dx
c
         cexx = -(
     *   + (y(i,j+1)-y(i+1,j))
     *   *(ZKXX(i+1,j+1)/(2.d0*VN(i+1,j+1)))*
     *    (y(i+2,j+1)-y(i+1,j))
     *   -(y(i+1,j+1)-y(i,j))
     *   *(-ZKXX(i+1,j)/(2.d0*VN(i+1,j)))*
     *    (y(i+1,j+1)-y(i+2,j)))

c
c    group related to the approximation of term d(zkxy du/dy)/dx
c
         cexy = -(
     *   + (y(i,j+1)-y(i+1,j))
     *   *(-ZKXY(i+1,j+1)/(2.d0*VN(i+1,j+1)))*
     *    (x(i+2,j+1)-x(i+1,j)) 
     *   -(y(i+1,j+1)-y(i,j))
     *   *(+ZKXY(i+1,j)/(2.d0*VN(i+1,j)))*
     *    (x(i+1,j+1)-x(i+2,j)))
c
c
c    group related to the approximation of term d(zkxy du/dx)/dy
c
         ceyx = -(
     *   - (x(i,j+1)-x(i+1,j))
     *   *(ZKXY(i+1,j+1)/(2.d0*VN(i+1,j+1)))*
     *    (y(i+2,j+1)-y(i+1,j))
     *   +(x(i+1,j+1)-x(i,j))
     *   *(-ZKXY(i+1,j)/(2.d0*VN(i+1,j)))*
     *    (y(i+1,j+1)-y(i+2,j)))


c
c    group related to the approximation of term d(zkyy du/dy)/dy
c
         ceyy = -(
     *   - (x(i,j+1)-x(i+1,j))
     *   *(-ZKYY(i+1,j+1)/(2.d0*VN(i+1,j+1)))*
     *    (x(i+2,j+1)-x(i+1,j))
     *   +(x(i+1,j+1)-x(i,j))
     *   *(+ZKYY(i+1,j)/(2.d0*VN(i+1,j)))*
     *    (x(i+1,j+1)-x(i+2,j)))



      ce(i,j) = cexx + cexy + ceyx + ceyy

c
c     COEFFICIENT NE(I,J), North-East, CELL (I+1,J+1)
c
c
      if(i.eq.M-1.and.j.eq.N-1) then
c     There is no cell i+1,j+1 for i=M, j=N
      cne(i,j) = 0.
      goto 4444
      end if
c
c    group related to the approximation of term d(zkxx du/dx)/dx
c
      cnexx =
     *    -(
     *   + (y(i,j+1)-y(i+1,j))
     *   *(ZKXX(i+1,j+1)/(2.d0*VN(i+1,j+1)))*
     *    (y(i+1,j+2)-y(i+2,j+1)))
c
c    group related to the approximation of term d(zkxy du/dy)/dx
c
      cnexy =
     *    -(
     *   + (y(i,j+1)-y(i+1,j))
     *   *(-ZKXY(i+1,j+1)/(2.d0*VN(i+1,j+1)))*
     *    (x(i+1,j+2)-x(i+2,j+1)))


c
c    group related to the approximation of term d(zkxy du/dx)/dy
c
      cneyx =
     *    -(
     *   - (x(i,j+1)-x(i+1,j))
     *   *(ZKXY(i+1,j+1)/(2.d0*VN(i+1,j+1)))*
     *    (y(i+1,j+2)-y(i+2,j+1)))


c
c    group related to the approximation of term d(zkyy du/dy)/dy
c
      cneyy =
     *    -(
     *   - (x(i,j+1)-x(i+1,j))
     *   *(-ZKYY(i+1,j+1)/(2.d0*VN(i+1,j+1)))*
     *    (x(i+1,j+2)-x(i+2,j+1)))


      cne(i,j) = cnexx + cnexy + cneyx + cneyy
4444  continue

c
c     COEFFICIENT N(I,J), North, CELL (I,J+1)
c
c    group related to the approximation of term d(zkxx du/dx)/dx
c
         cnxx =
     *  -(
     *   + (y(i,j+1)-y(i+1,j))
     *   *(ZKXX(i+1,j+1)/(2.d0*VN(i+1,j+1)))*
     *    (y(i,j+1)-y(i+1,j+2))
     *   -(y(i+1,j+1)-y(i,j))
     *   *(ZKXX(i,j+1)/(2.d0*VN(i,j+1)))*
     *    (y(i,j+2)-y(i+1,j+1)))

c
c    group related to the approximation of term d(zkxy du/dy)/dx
c
         cnxy =
     *  -(
     *   + (y(i,j+1)-y(i+1,j))
     *   *(-ZKXY(i+1,j+1)/(2.d0*VN(i+1,j+1)))*
     *    (x(i,j+1)-x(i+1,j+2))
     *   -(y(i+1,j+1)-y(i,j))
     *   *(-ZKXY(i,j+1)/(2.d0*VN(i,j+1)))*
     *    (x(i,j+2)-x(i+1,j+1)))
c
c    group related to the approximation of term d(zkxy du/dx)/dy
c
         cnyx =
     *  -(
     *   - (x(i,j+1)-x(i+1,j))
     *   *(ZKXY(i+1,j+1)/(2.d0*VN(i+1,j+1)))*
     *    (y(i,j+1)-y(i+1,j+2))
     *   +(x(i+1,j+1)-x(i,j))
     *   *(ZKXY(i,j+1)/(2.d0*VN(i,j+1)))*
     *    (y(i,j+2)-y(i+1,j+1)))
c
c    group related to the approximation of term d(zkyy du/dy)/dy
c
         cnyy =
     *  -(
     *   - (x(i,j+1)-x(i+1,j))
     *   *(-ZKYY(i+1,j+1)/(2.d0*VN(i+1,j+1)))*
     *    (x(i,j+1)-x(i+1,j+2))
     *   +(x(i+1,j+1)-x(i,j))
     *   *(-ZKYY(i,j+1)/(2.d0*VN(i,j+1)))*
     *    (x(i,j+2)-x(i+1,j+1)))

      cn(i,j) = cnxx + cnxy + cnyx + cnyy


c
c     COEFFICIENT NW(I,J), North-West, CELL (I-1,J+1)
c
c
       if(i.eq.1.and.j.eq.N-1) then
c      There is no cell i-1,j+1 for i=1, j=N
       cnw(i,j) = 0.
       goto 3333
       end if
c
c    group related to the approximation of term d(zkxx du/dx)/dx
c
      cnwxx =
     *  -(
     *   - (y(i+1,j+1)-y(i,j))
     *   *(ZKXX(i,j+1)/(2.d0*VN(i,j+1)))*
     *    (y(i-1,j+1)-y(i,j+2)))

c
c    group related to the approximation of term d(zkxy du/dy)/dx
c
      cnwxy =
     *  -(
     *   - (y(i+1,j+1)-y(i,j))
     *   *(-ZKXY(i,j+1)/(2.d0*VN(i,j+1)))*
     *    (x(i-1,j+1)-x(i,j+2)))


c
c    group related to the approximation of term d(zkxy du/dx)/dy
c
      cnwyx =
     *  -(
     *   + (x(i+1,j+1)-x(i,j))
     *   *(ZKXY(i,j+1)/(2.d0*VN(i,j+1)))*
     *    (y(i-1,j+1)-y(i,j+2)))


c
c    group related to the approximation of term d(zkyy du/dy)/dy
c
      cnwyy =
     *  -(
     *   + (x(i+1,j+1)-x(i,j))
     *   *(-ZKYY(i,j+1)/(2.d0*VN(i,j+1)))*
     *    (x(i-1,j+1)-x(i,j+2)))


      cnw(i,j) = cnwxx + cnwxy + cnwyx + cnwyy
3333  continue

c
c     COEFFICIENT W(I,J), West, CELL (I-1,J)
c
c    group related to the approximation of term d(zkxx du/dx)/dx
c
         cwxx =
     *  -(
     *   + (y(i,j+1)-y(i+1,j))
     *   *(-ZKXX(i,j)/(2.d0*VN(i,j)))*
     *    (y(i-1,j)-y(i,j+1))
     *   -(y(i+1,j+1)-y(i,j))
     *   *(ZKXX(i,j+1)/(2.d0*VN(i,j+1)))*
     *    (y(i,j)-y(i-1,j+1)))
c
c    group related to the approximation of term d(zkxy du/dy)/dx
c
         cwxy =
     *  -(
     *   + (y(i,j+1)-y(i+1,j))
     *   *(+ZKXY(i,j)/(2.d0*VN(i,j)))*
     *    (x(i-1,j)-x(i,j+1))
     *   -(y(i+1,j+1)-y(i,j))
     *   *(-ZKXY(i,j+1)/(2.d0*VN(i,j+1)))*
     *    (x(i,j)-x(i-1,j+1)))
c
c    group related to the approximation of term d(zkxy du/dx)/dy
c
         cwyx =
     *  -(
     *   - (x(i,j+1)-x(i+1,j))
     *   *(-ZKXY(i,j)/(2.d0*VN(i,j)))*
     *    (y(i-1,j)-y(i,j+1))
     *   +(x(i+1,j+1)-x(i,j))
     *   *(ZKXY(i,j+1)/(2.d0*VN(i,j+1)))*
     *    (y(i,j)-y(i-1,j+1)))
c
c    group related to the approximation of term d(zkyy du/dy)/dy
c
         cwyy =
     *  -(
     *   - (x(i,j+1)-x(i+1,j))
     *   *(+ZKYY(i,j)/(2.d0*VN(i,j)))*
     *    (x(i-1,j)-x(i,j+1))
     *   +(x(i+1,j+1)-x(i,j))
     *   *(-ZKYY(i,j+1)/(2.d0*VN(i,j+1)))*
     *    (x(i,j)-x(i-1,j+1)))

      cw(i,j) = cwxx + cwxy + cwyx + cwyy

c
c     COEFFICIENT SW(I,J), Sowth-West, CELL (I-1,J-1)
c
c
      if(i.eq.1.and.j.eq.1) then 
c     There is no cell (i-1,j-1) for i=1,j=1.
      csw(i,j) = 0.
      goto 1111
      end if
c
c
c    group related to the approximation of term d(zkxx du/dx)/dx
c
      cswxx =
     *   -(
     *   +(y(i,j+1)-y(i+1,j))
     *   *(-ZKXX(i,j)/(2.d0*VN(i,j)))*
     *    (y(i,j-1)-y(i-1,j)))

c
c    group related to the approximation of term d(zkxy du/dy)/dx
c
      cswxy =
     *   -(
     *   +(y(i,j+1)-y(i+1,j))
     *   *(+ZKXY(i,j)/(2.d0*VN(i,j)))*
     *    (x(i,j-1)-x(i-1,j)))


c
c    group related to the approximation of term d(zkxy du/dx)/dy
c
      cswyx =
     *   -(
     *   -(x(i,j+1)-x(i+1,j))
     *   *(-ZKXY(i,j)/(2.d0*VN(i,j)))*
     *    (y(i,j-1)-y(i-1,j)))


c
c    group related to the approximation of term d(zkyy du/dy)/dy
c
      cswyy =
     *   -(
     *   -(x(i,j+1)-x(i+1,j))
     *   *(+ZKYY(i,j)/(2.d0*VN(i,j)))*
     *    (x(i,j-1)-x(i-1,j)))


      csw(i,j) = cswxx + cswxy + cswyx + cswyy

1111  continue     
c
c     COEFFICIENT S(I,J), South, CELL (I,J-1)
c
c    group related to the approximation of term d(zkxx du/dx)/dx
c
         csxx =
     *  -(
     *   + (y(i,j+1)-y(i+1,j))
     *   *(-ZKXX(i,j)/(2.d0*VN(i,j)))*
     *    (y(i+1,j)-y(i,j-1))
     *   -(y(i+1,j+1)-y(i,j))
     *   *(-ZKXX(i+1,j)/(2.d0*VN(i+1,j)))*
     *    (y(i+1,j-1)-y(i,j)))
c
c    group related to the approximation of term d(zkxy du/dy)/dx
c
         csxy =
     *  -(
     *   + (y(i,j+1)-y(i+1,j))
     *   *(+ZKXY(i,j)/(2.d0*VN(i,j)))*
     *    (x(i+1,j)-x(i,j-1))
     *   -(y(i+1,j+1)-y(i,j))
     *   *(+ZKXY(i+1,j)/(2.d0*VN(i+1,j)))*
     *    (x(i+1,j-1)-x(i,j)))
c
c    group related to the approximation of term d(zkxy du/dx)/dy
c
         csyx =
     *  -(
     *   - (x(i,j+1)-x(i+1,j))
     *   *(-ZKXY(i,j)/(2.d0*VN(i,j)))*
     *    (y(i+1,j)-y(i,j-1))
     *   +(x(i+1,j+1)-x(i,j))
     *   *(-ZKXY(i+1,j)/(2.d0*VN(i+1,j)))*
     *    (y(i+1,j-1)-y(i,j)))
c
c    group related to the approximation of term d(zkyy du/dy)/dy
c
         csyy =
     *  -(
     *   - (x(i,j+1)-x(i+1,j))
     *   *(+ZKYY(i,j)/(2.d0*VN(i,j)))*
     *    (x(i+1,j)-x(i,j-1))
     *   +(x(i+1,j+1)-x(i,j))
     *   *(+ZKYY(i+1,j)/(2.d0*VN(i+1,j)))*
     *    (x(i+1,j-1)-x(i,j)))

      cs(i,j) = csxx + csxy + csyx + csyy



c
c     COEFFICIENT SE(I,J), South-East, CELL (I+1,J-1)
c
c
       if(i.eq.M-1.and.j.eq.1) then
c      There is no cell i+1,j-1 for i=M,j=1
       cse(i,j) = 0.
       goto 2222
       end if 
c
c    group related to the approximation of term d(zkxx du/dx)/dx
c
      csexx =
     *   -(
     *   -(y(i+1,j+1)-y(i,j))
     *   *(-ZKXX(i+1,j)/(2.d0*VN(i+1,j)))*
     *    (y(i+2,j)-y(i+1,j-1)))

c
c    group related to the approximation of term d(zkxy du/dy)/dx
c
      csexy =
     *   -(
     *   -(y(i+1,j+1)-y(i,j))
     *   *(+ZKXY(i+1,j)/(2.d0*VN(i+1,j)))*
     *    (x(i+2,j)-x(i+1,j-1)))


c
c    group related to the approximation of term d(zkxy du/dx)/dy
c
      cseyx =
     *   -(
     *   +(x(i+1,j+1)-x(i,j))
     *   *(-ZKXY(i+1,j)/(2.d0*VN(i+1,j)))*
     *    (y(i+2,j)-y(i+1,j-1)))


c
c    group related to the approximation of term d(zkyy du/dy)/dy
c
      cseyy =
     *   -(
     *   +(x(i+1,j+1)-x(i,j))
     *   *(+ZKYY(i+1,j)/(2.d0*VN(i+1,j)))*
     *    (x(i+2,j)-x(i+1,j-1)))


      cse(i,j) = csexx + csexy + cseyx + cseyy
2222  continue
c
c      COEFFICIENT IN CELL (I,J) - CC(i,j)


      cc(i,j) = - ( ce(i,j) + cne(i,j) + cn(i,j) + cnw(i,j) +
     *          cw(i,j) + csw(i,j) + cs(i,j) + cse(i,j))


      phi(i,j)=f(i,j) * ( 2.0d0 * VC(i,j) )
      end do
      end do

c===========================================================
c     computing the coefficients of the finite-difference scheme
c     and the right-hand side 

c     Approximation of the Robin boundary condition
c--------------------------------------------------

c     South-bottom boundary j=0  
c-------------------------------

      j=1
   
c     j=1 to simplify the code

      do i=1,M-1

c    Coefficient CN  
      cn(i,0) =
     * 0.5*(y(i+1,1)-y(i,1))*
     *((ZKXX(i,j)*(y(i,j+1)-y(i+1,j))
     * -ZKXY(i,j)*(x(i,j+1)-x(i+1,j)))/(2.*VN(i,j))
     * +(ZKXX(i+1,j)*(y(i,j)-y(i+1,j+1))
     * -ZKXY(i+1,j)*(x(i,j)-x(i+1,j+1)))/(2.*VN(i+1,j)))
     * -
     * 0.5*(x(i+1,1)-x(i,1))*
     * ((ZKXY(i,j)*(y(i,j+1)-y(i+1,j))
     * -ZKYY(i,j)*(x(i,j+1)-x(i+1,j)))/(2.*VN(i,j))
     * +(ZKXY(i+1,j)*(y(i,j)-y(i+1,j+1))
     * -ZKYY(i+1,j)*(x(i,j)-x(i+1,j+1)))/(2.*VN(i+1,j)))

c    Coefficient NE
       cne(i,0)=
     * 0.5*(y(i+1,1)-y(i,1))*
     * ((ZKXX(i+1,j)*(y(i+1,j+1)-y(i+2,j))
     * -ZKXY(i+1,j)*(x(i+1,j+1)-x(i+2,j)))/(2.*VN(i+1,j)))
     * -
     * 0.5*(x(i+1,1)-x(i,1))*
     * ((ZKXY(i+1,j)*(y(i+1,j+1)-y(i+2,j))
     * -ZKYY(i+1,j)*(x(i+1,j+1)-x(i+2,j)))/(2.*VN(i+1,j)))

c    Coefficient  NW
       cnw(i,0)=
     * 0.5*(y(i+1,1)-y(i,1))*
     * ((ZKXX(i,j)*(y(i-1,j)-y(i,j+1))
     * -ZKXY(i,j)*(x(i-1,j)-x(i,j+1)))/(2.*VN(i,j)))
     *  -
     * 0.5*(x(i+1,1)-x(i,1))*
     * ((ZKXY(i,j)*(y(i-1,j)-y(i,j+1))
     * -ZKYY(i,j)*(x(i-1,j)-x(i,j+1)))/(2.*VN(i,j)))

c    Coefficient W
       if(i.ne.1) then
       cw(i,0)=
     * 0.5*(y(i+1,1)-y(i,1))*
     * ((ZKXX(i,j)*(y(i,j-1)-y(i-1,j))
     * -ZKXY(i,j)*(x(i,j-1)-x(i-1,j)))/(2.*VN(i,j)))
     * -
     * 0.5*(x(i+1,1)-x(i,1))*
     * ((ZKXY(i,j)*(y(i,j-1)-y(i-1,j))
     * -ZKYY(i,j)*(x(i,j-1)-x(i-1,j)))/(2.*VN(i,j)))
        else
       cw(i,0)=0.
       end if

c    Coefficient C 
       cc(i,0)=
     * 0.5*(y(i+1,1)-y(i,1))*
     * ((ZKXX(i,j)*(y(i+1,j)-y(i,j-1))
     * -ZKXY(i,j)*(x(i+1,j)-x(i,j-1)))/(2.*VN(i,j)))
     * -
     * 0.5*(x(i+1,1)-x(i,1))*
     * ((ZKXY(i,j)*(y(i+1,j)-y(i,j-1))
     * -ZKYY(i,j)*(x(i+1,j)-x(i,j-1)))/(2.*VN(i,j)))
     * +
     * 0.5*(y(i+1,1)-y(i,1))*
     * ((ZKXX(i+1,j)*(y(i+1,j-1)-y(i,j))
     * -ZKXY(i+1,j)*(x(i+1,j-1)-x(i,j)))/(2.*VN(i+1,j)))
     * -
     * 0.5*(x(i+1,1)-x(i,1))*
     * ((ZKXY(i+1,j)*(y(i+1,j-1)-y(i,j))
     * -ZKYY(i+1,j)*(x(i+1,j-1)-x(i,j)))/(2.*VN(i+1,j)))
     * +
     * alphas(i)*zls(i)

c    Coefficient E
       if(i.ne.M-1) then 
       ce(i,0)=
     * 0.5*(y(i+1,1)-y(i,1))*
     * ((ZKXX(i+1,j)*(y(i+2,j)-y(i+1,j-1))
     * -ZKXY(i+1,j)*(x(i+2,j)-x(i+1,j-1)))/(2.*VN(i+1,j)))
     *  -
     * 0.5*(x(i+1,1)-x(i,1))*
     * ((ZKXY(i+1,j)*(y(i+2,j)-y(i+1,j-1))
     * -ZKYY(i+1,j)*(x(i+2,j)-x(i+1,j-1)))/(2.*VN(i+1,j)))
        else
        ce(i,0)=0.
        end if
       csw(i,0)=0.
       cs(i,0)=0.
       cse(i,0)=0.
         
      phi(i,0)=gammas(i) * zls(i)                
      end do
      


c     North-top boundary j=N  
c---------------------------

      j=N   

      do i=1,M-1

c    Coefficient C 
        cc(i,N)=     
     * -0.5*(y(i+1,N)-y(i,N))*
     * ((ZKXX(i,j)*(y(i,j+1)-y(i+1,j))
     * -ZKXY(i,j)*(x(i,j+1)-x(i+1,j)))/(2.*VN(i,j))
     * +
     * (ZKXX(i+1,j)*(y(i,j)-y(i+1,j+1))
     * -ZKXY(i+1,j)*(x(i,j)-x(i+1,j+1)))/(2.*VN(i+1,j)))
     * +
     * 0.5*(x(i+1,N)-x(i,N))*
     * ((ZKXY(i,j)*(y(i,j+1)-y(i+1,j))
     * -ZKYY(i,j)*(x(i,j+1)-x(i+1,j)))/(2.*VN(i,j))
     * +
     * (ZKXY(i+1,j)*(y(i,j)-y(i+1,j+1))
     * -ZKYY(i+1,j)*(x(i,j)-x(i+1,j+1)))/(2.*VN(i+1,j)))
     * +
     * alphan(i)*zln(i)

c    Coefficient E
        if(i.ne.M-1) then
        ce(i,N)=
     * -0.5*(y(i+1,N)-y(i,N))*
     * ((ZKXX(i+1,j)*(y(i+1,j+1)-y(i+2,j))
     * -ZKXY(i+1,j)*(x(i+1,j+1)-x(i+2,j)))/(2.*VN(i+1,j)))
     * +
     * 0.5*(x(i+1,N)-x(i,N))*
     * ((ZKXY(i+1,j)*(y(i+1,j+1)-y(i+2,j))
     * -ZKYY(i+1,j)*(x(i+1,j+1)-x(i+2,j)))/(2.*VN(i+1,j)))
        else
        ce(i,N)=0.
        end if

c    Coefficient W
       if(i.ne.1) then
       cw(i,N)=
     * -0.5*(y(i+1,N)-y(i,N))*
     * ((ZKXX(i,j)*(y(i-1,j)-y(i,j+1))
     * -ZKXY(i,j)*(x(i-1,j)-x(i,j+1)))/(2.*VN(i,j)))
     *  +
     * 0.5*(x(i+1,N)-x(i,N))*
     * ((ZKXY(i,j)*(y(i-1,j)-y(i,j+1))
     * -ZKYY(i,j)*(x(i-1,j)-x(i,j+1)))/(2.*VN(i,j)))
       else
       cw(i,N)=0.
       end if
   

c    Coefficient SW
        csw(i,N)=
     * -0.5*(y(i+1,N)-y(i,N))*
     * ((ZKXX(i,j)*(y(i,j-1)-y(i-1,j))
     * -ZKXY(i,j)*(x(i,j-1)-x(i-1,j)))/(2.*VN(i,j)))
     * +
     * 0.5*(x(i+1,N)-x(i,N))*
     * ((ZKXY(i,j)*(y(i,j-1)-y(i-1,j))
     * -ZKYY(i,j)*(x(i,j-1)-x(i-1,j)))/(2.*VN(i,j)))
     
c    Coefficient S
       cs(i,N)=
     * -0.5*(y(i+1,N)-y(i,N))*
     * ((ZKXX(i,j)*(y(i+1,j)-y(i,j-1))
     * -ZKXY(i,j)*(x(i+1,j)-x(i,j-1)))/(2.*VN(i,j)))
     * +
     * 0.5*(x(i+1,N)-x(i,N))*
     * ((ZKXY(i,j)*(y(i+1,j)-y(i,j-1))
     * -ZKYY(i,j)*(x(i+1,j)-x(i,j-1)))/(2.*VN(i,j)))
     * -
     * 0.5*(y(i+1,N)-y(i,N))*
     * ((ZKXX(i+1,j)*(y(i+1,j-1)-y(i,j))
     * -ZKXY(i+1,j)*(x(i+1,j-1)-x(i,j)))/(2.*VN(i+1,j)))
     * +
     * 0.5*(x(i+1,N)-x(i,N))*
     * ((ZKXY(i+1,j)*(y(i+1,j-1)-y(i,j))
     * -ZKYY(i+1,j)*(x(i+1,j-1)-x(i,j)))/(2.*VN(i+1,j)))

c    Coefficient SE
       cse(i,N)=
     * -0.5*(y(i+1,N)-y(i,N))*
     * ((ZKXX(i+1,j)*(y(i+2,j)-y(i+1,j-1))
     * -ZKXY(i+1,j)*(x(i+2,j)-x(i+1,j-1)))/(2.*VN(i+1,j)))
     *  +
     * 0.5*(x(i+1,N)-x(i,N))*
     * ((ZKXY(i+1,j)*(y(i+2,j)-y(i+1,j-1))
     * -ZKYY(i+1,j)*(x(i+2,j)-x(i+1,j-1)))/(2.*VN(i+1,j)))

      cne(i,N)=0.
      cn(i,N)=0.
      cnw(i,N)=0.

      phi(i,N)=gamman(i) * zln(i)  
              
      end do
	
c     East-right boundary i=M 
c----------------------------
      i=M    

      do j=1,N-1

c    Coefficient C
        cc(M,j)=
     * +0.5*(y(M,j+1)-y(M,j))*
     * ((ZKXX(i,j)*(y(i,j+1)-y(i+1,j))
     * -ZKXY(i,j)*(x(i,j+1)-x(i+1,j)))/(2.*VN(i,j))
     * +
     * (ZKXX(i,j+1)*(y(i+1,j+1)-y(i,j))
     * -ZKXY(i,j+1)*(x(i+1,j+1)-x(i,j)))/(2.*VN(i,j+1)))
     * -
     * 0.5*(x(M,j+1)-x(M,j))*
     * ((ZKXY(i,j)*(y(i,j+1)-y(i+1,j))
     * -ZKYY(i,j)*(x(i,j+1)-x(i+1,j)))/(2.*VN(i,j))
     * +
     * (ZKXY(i,j+1)*(y(i+1,j+1)-y(i,j))
     * -ZKYY(i,j+1)*(x(i+1,j+1)-x(i,j)))/(2.*VN(i,j+1)))
     * +
     * alphae(j)*zle(j)

c    Coefficient N
        if(j.ne.N-1) then
        cn(M,j)=
     * +0.5*(y(M,j+1)-y(M,j))*
     * (ZKXX(i,j+1)*(y(i,j+2)-y(i+1,j+1))
     * -ZKXY(i,j+1)*(x(i,j+2)-x(i+1,j+1)))/(2.*VN(i,j+1))
     * -
     * 0.5*(x(M,j+1)-x(M,j))*
     * (ZKXY(i,j+1)*(y(i,j+2)-y(i+1,j+1))
     * -ZKYY(i,j+1)*(x(i,j+2)-x(i+1,j+1)))/(2.*VN(i,j+1))
        else
        cn(M,j)=0.
        end if

c    Coefficient NW
        cnw(M,j)=
     * +0.5*(y(M,j+1)-y(M,j))*
     * (ZKXX(i,j+1)*(y(i-1,j+1)-y(i,j+2))
     * -ZKXY(i,j+1)*(x(i-1,j+1)-x(i,j+2)))/(2.*VN(i,j+1))
     * -
     * 0.5*(x(M,j+1)-x(M,j))*
     * (ZKXY(i,j+1)*(y(i-1,j+1)-y(i,j+2))
     * -ZKYY(i,j+1)*(x(i-1,j+1)-x(i,j+2)))/(2.*VN(i,j+1))

c    Coefficient W
         cw(M,j)=
     * +0.5*(y(M,j+1)-y(M,j))*
     * ((ZKXX(i,j)*(y(i-1,j)-y(i,j+1))
     * -ZKXY(i,j)*(x(i-1,j)-x(i,j+1)))/(2.*VN(i,j))
     * +
     * (ZKXX(i,j+1)*(y(i,j)-y(i-1,j+1))
     * -ZKXY(i,j+1)*(x(i,j)-x(i-1,j+1)))/(2.*VN(i,j+1)))
     * -
     * 0.5*(x(M,j+1)-x(M,j))*
     * ((ZKXY(i,j)*(y(i-1,j)-y(i,j+1))
     * -ZKYY(i,j)*(x(i-1,j)-x(i,j+1)))/(2.*VN(i,j))
     * +
     * (ZKXY(i,j+1)*(y(i,j)-y(i-1,j+1))
     * -ZKYY(i,j+1)*(x(i,j)-x(i-1,j+1)))/(2.*VN(i,j+1)))

c    Coefficient SW
        csw(M,j)=
     * +0.5*(y(M,j+1)-y(M,j))*
     * (ZKXX(i,j)*(y(i,j-1)-y(i-1,j))
     * -ZKXY(i,j)*(x(i,j-1)-x(i-1,j)))/(2.*VN(i,j))
     * -
     * 0.5*(x(M,j+1)-x(M,j))*
     * (ZKXY(i,j)*(y(i,j-1)-y(i-1,j))
     * -ZKYY(i,j)*(x(i,j-1)-x(i-1,j)))/(2.*VN(i,j))
     
c    Coefficient S
         if(j.ne.1) then
        cs(M,j)=
     * +0.5*(y(M,j+1)-y(M,j))*
     * (ZKXX(i,j)*(y(i+1,j)-y(i,j-1))
     * -ZKXY(i,j)*(x(i+1,j)-x(i,j-1)))/(2.*VN(i,j))
     * -
     * 0.5*(x(M,j+1)-x(M,j))*
     * (ZKXY(i,j)*(y(i+1,j)-y(i,j-1))
     * -ZKYY(i,j)*(x(i+1,j)-x(i,j-1)))/(2.*VN(i,j))
        else
        cs(M,j)=0.
        end if

       cne(M,j)=0.
       ce(M,j)=0.
       cse(M,j)=0.

      phi(M,j)=gammae(j) * zle(j)
                
       end do

c     West-left boundary i=0
c----------------------------
      i=1     
c     i=1 for simplicity of the formulas

      do j=1,N-1

c    Coefficient E
        ce(0,j)=
     * -0.5*(y(1,j+1)-y(1,j))*
     * ((ZKXX(i,j)*(y(i,j+1)-y(i+1,j))
     * -ZKXY(i,j)*(x(i,j+1)-x(i+1,j)))/(2.*VN(i,j))
     * +
     * (ZKXX(i,j+1)*(y(i+1,j+1)-y(i,j))
     * -ZKXY(i,j+1)*(x(i+1,j+1)-x(i,j)))/(2.*VN(i,j+1)))
     * +
     * 0.5*(x(1,j+1)-x(1,j))*
     * ((ZKXY(i,j)*(y(i,j+1)-y(i+1,j))
     * -ZKYY(i,j)*(x(i,j+1)-x(i+1,j)))/(2.*VN(i,j))
     * +
     * (ZKXY(i,j+1)*(y(i+1,j+1)-y(i,j))
     * -ZKYY(i,j+1)*(x(i+1,j+1)-x(i,j)))/(2.*VN(i,j+1)))

c    Coefficient NE
        cne(0,j)=
     * -0.5*(y(1,j+1)-y(1,j))*
     * (ZKXX(i,j+1)*(y(i,j+2)-y(i+1,j+1))
     * -ZKXY(i,j+1)*(x(i,j+2)-x(i+1,j+1)))/(2.*VN(i,j+1))
     * +
     * 0.5*(x(1,j+1)-x(1,j))*
     * (ZKXY(i,j+1)*(y(i,j+2)-y(i+1,j+1))
     * -ZKYY(i,j+1)*(x(i,j+2)-x(i+1,j+1)))/(2.*VN(i,j+1))

c    Coefficient N
        if(j.ne.N-1) then
        cn(0,j)=
     * -0.5*(y(1,j+1)-y(1,j))*
     * (ZKXX(i,j+1)*(y(i-1,j+1)-y(i,j+2))
     * -ZKXY(i,j+1)*(x(i-1,j+1)-x(i,j+2)))/(2.*VN(i,j+1))
     * +
     * 0.5*(x(1,j+1)-x(1,j))*
     * (ZKXY(i,j+1)*(y(i-1,j+1)-y(i,j+2))
     * -ZKYY(i,j+1)*(x(i-1,j+1)-x(i,j+2)))/(2.*VN(i,j+1))
        else
        cn(0,j)=0.
        end if

c    Coefficient C
         cc(0,j)=
     * -0.5*(y(1,j+1)-y(1,j))*
     * ((ZKXX(i,j)*(y(i-1,j)-y(i,j+1))
     * -ZKXY(i,j)*(x(i-1,j)-x(i,j+1)))/(2.*VN(i,j))
     * +
     * (ZKXX(i,j+1)*(y(i,j)-y(i-1,j+1))
     * -ZKXY(i,j+1)*(x(i,j)-x(i-1,j+1)))/(2.*VN(i,j+1)))
     * +
     * 0.5*(x(1,j+1)-x(1,j))*
     * ((ZKXY(i,j)*(y(i-1,j)-y(i,j+1))
     * -ZKYY(i,j)*(x(i-1,j)-x(i,j+1)))/(2.*VN(i,j))
     * +
     * (ZKXY(i,j+1)*(y(i,j)-y(i-1,j+1))
     * -ZKYY(i,j+1)*(x(i,j)-x(i-1,j+1)))/(2.*VN(i,j+1)))
     *+ alphaw(j)*zlw(j)

c    Coefficient S
         if(j.ne.1) then
        cs(0,j)=
     * -0.5*(y(1,j+1)-y(1,j))*
     * (ZKXX(i,j)*(y(i,j-1)-y(i-1,j))
     * -ZKXY(i,j)*(x(i,j-1)-x(i-1,j)))/(2.*VN(i,j))
     * +
     * 0.5*(x(1,j+1)-x(1,j))*
     * (ZKXY(i,j)*(y(i,j-1)-y(i-1,j))
     * -ZKYY(i,j)*(x(i,j-1)-x(i-1,j)))/(2.*VN(i,j))
        else
        cs(0,j)=0.
        end if
     
c    Coefficient SE
        cse(0,j)=
     * -0.5*(y(1,j+1)-y(1,j))*
     * (ZKXX(i,j)*(y(i+1,j)-y(i,j-1))
     * -ZKXY(i,j)*(x(i+1,j)-x(i,j-1)))/(2.*VN(i,j))
     * +
     * 0.5*(x(1,j+1)-x(1,j))*
     * (ZKXY(i,j)*(y(i+1,j)-y(i,j-1))
     * -ZKYY(i,j)*(x(i+1,j)-x(i,j-1)))/(2.*VN(i,j))

       csw(0,j)=0.
       cw(0,j)=0.
       cnw(0,j)=0.

      phi(0,j)=gammaw(j) * zlw(j)  
              
      end do

      return
      end
  
c**************************************************************

      subroutine siedel
c
c     SOLUTION for the system of linear algebraic equations
c     by the GAUSS-SIEDEL iteration method
c
      implicit real*8 (a-h,o-z)

c     M,N - number of nodes in the grid 
      common/MN/ M,N

c     X,Y - arrays (0 to M+1 and 0 to N+1), coordinates of grid nodes
c     X and Y for i=0,M+1; j=0,N+1 are fictitious, and are used
c     for making the computational formulas and code shorter.
      common/xy/ x(0:201,0:201),y(0:201,0:201)

c     u - array (0 to M and 0 to N), solution for the finite-difference scheme
      common/U/ U(0:200,0:200)
c    
c     =================================================================
c     CC,CE,CNE,CN,CNW,CW,CSW,CS,CSE, PHI - arrays of coefficients
c                                           and the right-hand side
c     ( 0 to M and 0 to N ) in finite-difference equations
c
c     E(i,j)*U(i+1,j) + NE(i,j)*U(i+1,j+1) + N(i,j)*U(i,j+1) +
c
c     NW(i,j)*U(i-1,j+1) + W(i,j)*U(i-1,j) +SW(i,j)*U(i-1,j-1) +
c
c     S(i,j)*U(i,j-1) + SE(i,j)*U(i+1,j-1) = PHI(i,j)
c


c----------------------  STENCIL --------------------------------
c
c
c
c              (i-1,j+2)     (i,j+2)         (i+1,j+2)         (i+2,j+2)
c             *-------------*---------------*-----------------*
c             !             !               !                 !
c             !             !               !                 !
c             !             !               !                 !                 
c             !    NW(i,j)  !   N(i,j)      !   NE(i,j)       !                 
c             !             !               !                 !
c             !             !               !                 ! 
c             !(i-1,j+1)    !(i,j+1)        !(i+1,j+1)        !(i+2,j+1)
c             *-------------*---------------*-----------------*
c             !             !               !                 !
c             !             !               !                 !
c             !             !               !                 !
c             !    W(i,j)   !   C(i,j)      !   E(i,j)        !
c             !             !               !                 !
c             !             !               !                 !
c             !(i-1,j)      !(i,j)          !(i+1,j)          !(i+2,j)
c             *-------------*---------------*-----------------*
c             !             !               !                 !
c             !             !               !                 !
c             !             !               !                 !
c             !    SW(i,j)  !   S(i,j)      !   SE(i,j)       !                 
c             !             !               !                 !
c             !             !               !                 !
c             !(i-1,j-1)    !(i,j-1)        !(i+1,j-1)        !(i+2,j-1)
c             *-------------*---------------*-----------------*
c
c
c
c
c
c
c----------------------  STENCIL --------------------------------

c
c
      common/coefds/
     *      cc(0:200,0:200),ce(0:200,0:200),
     *      cne(0:200,0:200),cn(0:200,0:200),
     *      cnw(0:200,0:200),cw(0:200,0:200),
     *      csw(0:200,0:200),cs(0:200,0:200),
     *      cse(0:200,0:200),phi(0:200,0:200)
c=================================================================
c      ALPHAS, ALPHAE, ALPHAN, ALPHAW - arrays which correspond
c     to function alpha(x,y) in the left-hand side of the boundary
c     conditions: - GIVEN in the middle of the boundary segments
c     ALPHAS, ALPHAN - south and north boundary - arrays (1 to M-1 )
c     ALPHAW, ALPHAE - west and east boundary - arrays ( 1 to N-1  )
      common/alpha/ alphas(200), alphae(200),
     * alphan(200), alphaw(200)

c     GAMMAS, GAMMAE, GAMMAN, GAMMAW - arrays which correspond
c     to function gamma(x,y) in the right-hand side of the
c     boundary conditions: - GIVEN in the middle of the boundary
c     GAMMAS, GAMMAN - south and north boundary - arrays (1 to M-1 )
c     GAMMAW, GAMMAE - west and east boundary - arrays ( 1 to N-1 )
      common/gamma/ gammas(200), gammae(200),
     * gamman(200), gammaw(200)


c=================================================================

c     eps - constant in the condition of finishing the iteration in 
c     the Gauss-Siedel iteration procedure.  The termination condition is
c
c             (S+1)     (S)
c     max( abs( U(i,j) - U(i,j))) < eps
c     i,j
c 
c     where S is the iteration number.
c

      common/eps/ eps
c=====================================================================
c     SET an INITIAL value for the iteration solution
      do  i=0, M
      do  j=0, N
      U(i,j)=0.d0
      end do
      end do

c     Beginning of the iteration process
c
5      continue

       EPSMAX= 0.D0
c
       do  i=0, M
       do  j=0, N
c      There are no equations for (0,0); (M,N);
c                                 (0,N); (M,0).
       if(i.eq.0.and.j.eq.0) goto 3333
       if(i.eq.M.and.j.eq.N) goto 3333
       if(i.eq.0.and.j.eq.N) goto 3333
       if(i.eq.M.and.j.eq.0) goto 3333

c      Checking: does indicies (it,jt) have a sense
c      for each cell in the stencil; If NOT, then formally
c      take it equal to 0 to keep the formulas the same.

       it=i+1 
       jt=j
       if(((it.eq.0).and.(jt.eq.0)).or.
     *    ((it.eq.0).and.(jt.eq.N)).or.  
     *    ((it.eq.M).and.(jt.eq.N)).or.  
     *    ((it.eq.M).and.(jt.eq.0))) then
       UE=0.
       else
       UE=U(it,jt)
       end if

       it=i+1 
       jt=j+1
       if(((it.eq.0).and.(jt.eq.0)).or.
     *    ((it.eq.0).and.(jt.eq.N)).or.  
     *    ((it.eq.M).and.(jt.eq.N)).or.  
     *    ((it.eq.M).and.(jt.eq.0))) then
       UNE=0.
       else
       UNE=U(it,jt)
       end if

       it=i 
       jt=j+1
       if(((it.eq.0).and.(jt.eq.0)).or.
     *    ((it.eq.0).and.(jt.eq.N)).or.  
     *    ((it.eq.M).and.(jt.eq.N)).or.  
     *    ((it.eq.M).and.(jt.eq.0))) then
       UN=0.
       else
       UN=U(it,jt)
       end if

       it=i-1 
       jt=j+1
       if(((it.eq.0).and.(jt.eq.0)).or.
     *    ((it.eq.0).and.(jt.eq.N)).or.  
     *    ((it.eq.M).and.(jt.eq.N)).or.  
     *    ((it.eq.M).and.(jt.eq.0))) then
       UNW=0.
       else
       UNW=U(it,jt)
       end if

       it=i-1 
       jt=j
       if(((it.eq.0).and.(jt.eq.0)).or.
     *    ((it.eq.0).and.(jt.eq.N)).or.  
     *    ((it.eq.M).and.(jt.eq.N)).or.  
     *    ((it.eq.M).and.(jt.eq.0))) then
       UW=0.
       else
       UW=U(it,jt)
       end if

       it=i-1 
       jt=j-1
       if(((it.eq.0).and.(jt.eq.0)).or.
     *    ((it.eq.0).and.(jt.eq.N)).or.  
     *    ((it.eq.M).and.(jt.eq.N)).or.  
     *    ((it.eq.M).and.(jt.eq.0))) then
       USW=0.
       else
       USW=U(it,jt)
       end if

       it=i 
       jt=j-1
       if(((it.eq.0).and.(jt.eq.0)).or.
     *    ((it.eq.0).and.(jt.eq.N)).or.  
     *    ((it.eq.M).and.(jt.eq.N)).or.  
     *    ((it.eq.M).and.(jt.eq.0))) then
       US=0.
       else
       US=U(it,jt)
       end if

       it=i+1 
       jt=j-1
       if(((it.eq.0).and.(jt.eq.0)).or.
     *    ((it.eq.0).and.(jt.eq.N)).or.  
     *    ((it.eq.M).and.(jt.eq.N)).or.  
     *    ((it.eq.M).and.(jt.eq.0))) then
       USE=0.
       else
       USE=U(it,jt)
       end if

c      Siedel iteration

       USP1=( phi(i,j)-
     * (CE(i,j)*UE + CNE(i,j)*UNE + CN(i,j)*UN +
     * CNW(i,j)*UNW + CW(i,j)*UW +CSW(i,j)*USW +
     * CS(i,j)*US + CSE(i,j)*USE))/CC(i,j)


       EPSMAX=amax1(EPSMAX,dabs(USP1 - U(i,j)))

       U(i,j)=USP1

3333   continue
       end do
       end do

c     checking the termination condition
       if(EPSMAX.GT.EPS) goto 5

       return
       end
c*******************************************************************
      
       subroutine mkgrid
c      This is a subroutine for generation of a smooth grid in 2D
c      as it is shown in the book.
c
c      Left boundary x=0,
c      Right boundary x=1,
c      Bottom boundary y=0,
c      Upper boundary y(x)=1.+0.5*sin(2.*pi*x)
c           _
c          / \
c         /   \
c        /     \
c       /       \
c      /         \
c      I          \                   /I
c      I           \                 / I
c      I            \               /  I
c      I             \             /   I
c      I              \           /    I
c      I               \         /     I
c      I                \       /      I
c      I                 \     /       I
c      I                  \   /        I
c      I                   \_/         I
c      I                               I
c      I_______________________________I
c
c
c      
c 

      implicit real*8 (a-h,o-z)

c     M,N - number of nodes in the grid 
      common/MN/ M,N

c     X,Y - arrays (0 to M+1 and 0 to N+1), coordinates of grid nodes
c     X and Y for i=0,M+1; j=0,N+1 are fictitious, and are used
c     for making the computational formulas and code shorter.
      common/xy/ x(0:201,0:201),y(0:201,0:201)

      M=17  

      N=17

      pi=acos(-1.)

      hx=1./(M-1.)

      do i=1, M
      do j=1, N
      xi=(i-1)*hx
      et=(j-1.)/(M-1.)  
      hyj=(1.+0.5*sin(2.*pi*xi))/(M-1.)
      x(i,j)=xi
      y(i,j)=(j-1.)*hyj 
      end do
      end do

c     Fictitious values of X and Y

      do i=1,M
      x(i,N+1) = x(i,N)
      y(i,N+1) = y(i,N)
      x(i,0) = x(i,1)
      y(i,0) = y(i,1)
      end do

      do j=1,N
      x(M+1,j) = x(M,j)
      y(M+1,j) = y(M,j)
      x(0,j) = x(1,j)
      y(0,j) = y(1,j)
      end do

      return
      end
c**********************************************************************

      subroutine exact
c     Compute the exact solution for the test problem
c     in cells and boundary sides.
c     Results in array
c     uex - 0 to M and 0 to N

      implicit real*8 (a-h,o-z)

c     M,N - number of nodes in the grid 
      common/MN/ M,N

c     X,Y - arrays (0 to M+1 and 0 to N+1), coordinates of grid nodes
c     X and Y for i=0,M+1; j=0,N+1 are fictitious, and are used
c     for making the computational formulas and program code shorter.
      common/xy/ x(0:201,0:201),y(0:201,0:201)

c     UEX - array 0 to M and 0 to N; Exact solution
      common/uex/ uex(0:200,0:200)

      pi=acos(-1.)

      do i=1,M-1
      do j=1,N-1
      xcij=0.25*(x(i,j)+x(i+1,j)+x(i+1,j+1)+x(i,j+1))
      ycij=0.25*(y(i,j)+y(i+1,j)+y(i+1,j+1)+y(i,j+1))
      uex(i,j)=sin(pi*xcij)*sin(pi*ycij)
      end do
      end do

      i=0
      do j=1,N-1
      xcij=0.5*(x(i,j)+x(i,j+1))
      ycij=0.5*(y(i,j)+y(i,j+1))
      uex(i,j)=sin(pi*xcij)*sin(pi*ycij)
      end do

      i=M
      do j=1,N-1
      xcij=0.5*(x(i,j)+x(i,j+1))
      ycij=0.5*(y(i,j)+y(i,j+1))
      uex(i,j)=sin(pi*xcij)*sin(pi*ycij)
      end do

      j=0
      do i=1,M-1
      xcij=0.5*(x(i,j)+x(i+1,j))
      ycij=0.5*(y(i,j)+y(i+1,j))
      uex(i,j)=sin(pi*xcij)*sin(pi*ycij)
      end do

      j=N
      do i=1,M-1
      xcij=0.5*(x(i,j)+x(i+1,j))
      ycij=0.5*(y(i,j)+y(i+1,j))
      uex(i,j)=sin(pi*xcij)*sin(pi*ycij)
      end do

      return
      end
c*****************************************************************

      subroutine output

c     Output subroutine 
c     prints the approximate solution,
c     exact solution and errors for
c     the test problem

      implicit real*8 (a-h,o-z)

c     M,N - number of nodes in the grid 
      common/MN/ M,N

c     X,Y - arrays (0 to M+1 and 0 to N+1), coordinates of grid nodes
c     X and Y for i=0,M+1; j=0,N+1 are fictitious, and are used
c     for making the computational formulas and program code shorter.
      common/xy/ x(0:201,0:201),y(0:201,0:201)

c     UEX - array 0 to M and 0 to N; Exact solution
      common/uex/ uex(0:200,0:200)

c     u - array (0 to M and 0 to N), solution for the finite-difference scheme
      common/U/ U(0:200,0:200)

c     ERR - array 0 to M and 0 to N; Error between the exact and
c           approximate solutions in nodes
      common/err/ err(0:200,0:200)

c     Max-norm and L2-norm of the error between the exact and approximate
c     solutions in nodes
      common/errors/supn,zL2N

c     Open the file p2drcn.output
c     for test output of program p2drcn
      open(unit=1,file='p2drcn.output')
      
      write(1,33) 
33    format(15X,'TEST OUTPUT FOR PROGRAM p2drcn')
      write(1,*)
      
      write(1,8) M,N, supn, zL2N
8     format(2X,'M=',I5,2X,'N=',I5,2X,
     *       'MAX-NORM = ',E12.5,2X,'  L2-NORM = ',E12.5)

       write(1,*)

c     Print of the approximate and exact solutions of the test problem
c     Print for I=I1,I2,I3 and for all J=1,...,N

      I1=2
      I2=9
      I3=16
      write(1,9) I1
9     format(10X,'Approximate and Exact Solutions, and Error for',
     * ' I=',I5)
       write(1,*)
      
      write(1,10)
10    format(5X,'j',7X,' X',12X,' Y',10X,'Appr.Sol.',
     * 4X,'Exact Sol.',5X,'Error')
       i=I1
      do j=0,N 
      xcij=0.25*(x(i,j)+x(i+1,j)+x(i+1,j+1)+x(i,j+1))
      ycij=0.25*(y(i,j)+y(i+1,j)+y(i+1,j+1)+y(i,j+1))
      write(1,11) j,xcij,ycij,u(i,j),uex(i,j),err(i,j)
      end do
       write(1,*)
       write(1,*)
11    format(1X, i5,5(2X,E12.5))

      write(1,9) I2
       write(1,*)
       i=I2
      do j=0,N 
      xcij=0.25*(x(i,j)+x(i+1,j)+x(i+1,j+1)+x(i,j+1))
      ycij=0.25*(y(i,j)+y(i+1,j)+y(i+1,j+1)+y(i,j+1))
      write(1,11) j,xcij,ycij,u(i,j),uex(i,j),err(i,j)
      end do
       write(1,*)
       write(1,*)

      write(1,9) I3
       write(1,*)
       i=I3
      do j=0,N 
      xcij=0.25*(x(i,j)+x(i+1,j)+x(i+1,j+1)+x(i,j+1))
      ycij=0.25*(y(i,j)+y(i+1,j)+y(i+1,j+1)+y(i,j+1))
      write(1,11) j,xcij,ycij,u(i,j),uex(i,j),err(i,j)
      end do
       write(1,*)
       write(1,*)

       return
       end

c*********************************************************************

      subroutine error
c     compute the Max and L2 norm of
c     error for the test problem
      implicit real*8 (a-h,o-z)

c     M,N - number of nodes in the grid 
      common/MN/ M,N

c     X,Y - arrays (0 to M+1 and 0 to N+1), coordinates of grid nodes
c     X and Y for i=0,M+1; j=0,N+1 are fictitious, and are used
c     for making the computational formulas and program code shorter.
      common/xy/ x(0:201,0:201),y(0:201,0:201)

c     UEX - array 0 to M and 0 to N; Exact solution
      common/uex/ uex(0:200,0:200)

c     u - array (0 to M and 0 to N), solution for the finite-difference scheme
      common/U/ U(0:200,0:200)

c     ERR - array 0 to M and 0 to N; Error between the exact and
c           approximate solutions in nodes
      common/err/ err(0:200,0:200)

c     Max-norm and L2-norm of the error between the exact and approximate
c     solutions in nodes
      common/errors/supn,zL2N

c     VN - array for the volume of nodes
      dimension VN(200,200),VC(200,200)
c     Comments - this volume is needed in this subroutine
c     only for computing the L2 norm of error	

c     ZLS, ZLE, ZLN, ZLW - arrays for the length of boundary sides
c                          on the boundaries
c     ZLS - south, ZLE - east, ZLN - north, ZLW - west.
      dimension zls(200), zle(200), zln(200), zlw(200)
c     Comments - these lengths are needed in this subroutine
c     only for computing the L2 norm of error	

c     Computing the exact solution
      call exact
c     
c     computing the volume of the cell VC(i,j)

      do  i=1,M-1
      do  j=1,N-1

      VC(i,j) =0.5*((x(i,j)-x(i+1,j+1))*(y(i+1,j)-y(i,j+1)) -
     *              (x(i+1,j)-x(i,j+1))*(y(i,j)-y(i+1,j+1)))

      end do
      end do

c     Computing the  volume of the nodes


      do i=1,M
      do j=1,N
c     Two different choices for VN(i,j)
c     First - average of volumes of adjacent cells
c     VN(i,j)=0.25*(VC(i,j) + VC(i-1,j) + VC(i-1,j-1) + VC(i,j-1))
c     Second "DIAMOND" volume
      VN(i,j)=0.25*((x(i-1,j)-x(i+1,j))*(y(i,j-1)-y(i,j+1)) -
     *              (x(i,j-1)-x(i,j+1))*(y(i-1,j)-y(i+1,j)))
c
      end do
      end do

      do i=1,M-1
      zls(i) = sqrt((x(i+1,1)-x(i,1))**2 +
     *              (y(i+1,1)-y(i,1))**2)

      zln(i) = sqrt((x(i+1,N)-x(i,N))**2 +
     *              (y(i+1,N)-y(i,N))**2)
      end do

c     East and West boundary 
      do j=1,N-1
      zle(j) = sqrt((x(M,j+1)-x(M,j))**2 +
     *              (y(M,j+1)-y(M,j))**2)

      zlw(j) = sqrt((x(1,j+1)-x(1,j))**2 +
     *              (y(1,j+1)-y(1,j))**2)
      end do

c     Error
c     -----

      do i=0,M
      do j=0,N
      err(i,j)=uex(i,j)-u(i,j)
      end do
      end do

c     Max-Norm
c     --------
      supn=0.d0
      do i=0,M
      do j=0,N
       if(i.eq.0.and.j.eq.0) goto 3333
       if(i.eq.M.and.j.eq.N) goto 3333
       if(i.eq.0.and.j.eq.N) goto 3333
       if(i.eq.M.and.j.eq.0) goto 3333
      supn=amax1(supn,abs(err(i,j)))
3333  continue
      end do
      end do
      
c     L  - norm
c      2
c     ----------

      zL2N=0.0d0
      do i=1,M-1
      do j=1,N-1
      zL2N = zL2N + err(i,j)**2*VC(i,j)
      end do
      end do
c    
c     Boundary terms

      do i=1,M-1
      zL2N = zL2N+err(i,0)**2*zls(i)
      zL2N = zL2N+err(i,N)**2*zln(i)
      end do

      do j=1,N-1
      zL2N = zL2N+err(0,j)**2*zlw(j)
      zL2N = zL2N+err(M,j)**2*zle(j)
      end do
                                 
      zL2N=dsqrt(zL2N)
      
      return
      end
c**********************************************************************

      subroutine bcon
c     This subroutine computes the values of
c     the finite-difference analogs of functions
c     alpha and gamma in Robin boundary conditions.
c     falphan(x,y),falphas(x,y),falphaw(x,y),falphaw(x,y)
c     and
c     fgamman(x,y),fgammas(x,y),fgammaw(x,y),fgammaw(x,y)
c     functions which compute the values of
c     functions alpha and gamma on
c     related parts of the boundary.  
     
      implicit real*8 (a-h,o-z)
c     M,N - number of nodes in the grid 
      common/MN/ M,N

c     X,Y - arrays (0 to M+1 and 0 to N+1), coordinates of grid nodes
c     X and Y for i=0,M+1; j=0,N+1 are fictitious, and are used
c     for making the computational formulas and program code shorter.
      common/xy/ x(0:201,0:201),y(0:201,0:201)
   
c      ALPHAS, ALPHAE, ALPHAN, ALPHAW - arrays which correspond
c     to function alpha(x,y) in the left-hand side of the boundary
c     conditions: - GIVEN in the middle of the boundary segments
c     ALPHAS, ALPHAN - south and north boundary - arrays (1 to M-1 )
c     ALPHAW, ALPHAE - west and east boundary - arrays ( 1 to N-1  )
      common/alpha/ alphas(200), alphae(200),
     * alphan(200), alphaw(200)

c     GAMMAS, GAMMAE, GAMMAN, GAMMAW - arrays which correspond
c     to function gamma(x,y) in the right-hand side of the
c     boundary conditions: - GIVEN in the middle of the boundary
c     GAMMAS, GAMMAN - south and north boundary - arrays (1 to M-1 )
c     GAMMAW, GAMMAE - west and east boundary - arrays ( 1 to N-1 )
      common/gamma/ gammas(200), gammae(200),
     * gamman(200), gammaw(200)


c     Set Robin boundary conditions
c     -----------------------------------

c     North and South Boundary
      do  i=1,M-1
      xciN=0.5*(x(i,N)+x(i+1,N))
      yciN=0.5*(y(i,N)+y(i+1,N))
      alphan(i) = falphan(xciN,yciN)
      gamman(i) = fgamman(xciN,yciN)

      xci1=0.5*(x(i,1)+x(i+1,1))
      yci1=0.5*(y(i,1)+y(i+1,1))
      alphas(i) = falphas(xci1,yci1)
      gammas(i) = fgammas(xci1,yci1)
      end do


c     East and West Boundary
      do  j=1,N
      xcMj=0.5*(x(M,j)+x(M,j+1))
      ycMj=0.5*(y(M,j)+y(M,j+1))
      alphae(j) = falphae(xcMj,ycMj)
      gammae(j) = fgammae(xcMj,ycMj)

      xc1j=0.5*(x(1,j)+x(1,j+1))
      yc1j=0.5*(y(1,j)+y(1,j+1))
      alphaw(j) = falphaw(xc1j,yc1j) 
      gammaw(j) = fgammaw(xc1j,yc1j) 

      end do

      return
      end

c***********************************************************************

      real function falphan*8(x,y)
c     function falpha - returned values of coefficient alpha
c     in the left-hand side of Robin boundary conditions of
c     a differential problem
c     North - Top - boundary -> j=N
      implicit real*8 (a-h,o-z)
      falphan=1.                    
      return
      end
c***********************************************************************

      real function falphas*8(x,y)
c     function falphas - returned values of coefficient alpha
c     in the left-hand side of Robin boundary conditions of
c     a differential problem
c     South - Bottom - boundary -> j=1
      implicit real*8 (a-h,o-z)
      falphas=1.                     
      return
      end
c***********************************************************************

      real function falphae*8(x,y)
c     function falphe - returned values of coefficient alpha
c     in the left-hand side of Robin boundary conditions of
c     a differential problem
c     East - Right - boundary -> i=M
      implicIt real*8 (a-h,o-z)
      falphae=1.                  
      return
      end
c***********************************************************************

      real function falphaw*8(x,y)
c     function falphaw - returned values of alpha
c     in the left-hand side of Robin boundary conditions of
c     a differential problem
c     West - left - boundary -> i=1
      implicit real*8 (a-h,o-z)
      falphaw=1.                    
      return
      end
c***********************************************************************

      real function fgamman*8(x,y)
c     function fgamman- returned values of coefficient gamma
c     in the right-hand side of Robin boundary conditions of
c     a differential problem
c     North - Top - boundary -> j=N
      implicit real*8 (a-h,o-z)

      pi=acos(-1.)

            d1  = 1.0d0 + 2.0d0*x**2 + y**2
            d2  = 1.0d0 + x**2 + 2.0d0*y**2
            d1x = 4.0d0*x
            d1y = 2.0d0*y
            d2x = 2.0d0*x
            d2y = 4.0d0*y
            t  = 5.d0*pi/12.d0
            ct = dcos(t)
            st = dsin(t)
            zkxx = d1*ct*ct + d2*st*st
            zkxy = (d1-d2)*ct*st
            zkyy = d1*st*st + d2*ct*ct
c
            sx  = sin(pi*x)
            cx  = cos(pi*x)
            sy  = sin(pi*y)
            cy  = cos(pi*y)
            fx       =     pi*cx*sy
            fy       =     pi*sx*cy
             fe=sx*sy

c     Components of unit outwards normal to curve
c     1+0.5*sin(2.*pi.*x)

      znx=-pi*cos(2.*pi*x)/sqrt(1.+pi**2*cos(2.*pi*x)**2)
      zny=+1./sqrt(1.+pi**2*cos(2.*pi*x)**2)
c
c     Components of vector K*GRAD f

      zkgx=zkxx*fx+zkxy*fy
      zkgy=zkxy*fx+zkyy*fy

      fgamman=znx*zkgx+zny*zkgy + fe

 
      return
      end
c***********************************************************************

      real function fgammas*8(x,y)
c     function fgammas - returned values of coefficient gamma
c     in the right-hand side of Robin boundary conditions of
c     the differential problem
c     South - Bottom - boundary -> j=1
      implicit real*8 (a-h,o-z)

      pi=acos(-1.)

            d1  = 1.0d0 + 2.0d0*x**2 + y**2
            d2  = 1.0d0 + x**2 + 2.0d0*y**2
            d1x = 4.0d0*x
            d1y = 2.0d0*y
            d2x = 2.0d0*x
            d2y = 4.0d0*y
            t  = 5.d0*pi/12.d0
            ct = dcos(t)
            st = dsin(t)
            zkxx = d1*ct*ct + d2*st*st
            zkxy = (d1-d2)*ct*st
            zkyy = d1*st*st + d2*ct*ct

c
            sx  = sin(pi*x)
            cx  = cos(pi*x)
            sy  = sin(pi*y)
            cy  = cos(pi*y)
            fx       =     pi*cx*sy
            fy       =     pi*sx*cy
            fe=sx*sy

      fgammas=-(zkxy*fx+zkyy*fy) + fe
 
      return
      end
c***********************************************************************

      real function fgammae*8(x,y)
c     function fgammae - returned values of coefficient gamma
c     in the right-hand side of Robin boundary conditions of
c     a differential problem
c     East - Right - boundary -> i=M
      implicIt real*8 (a-h,o-z)

      pi=acos(-1.)

            d1  = 1.0d0 + 2.0d0*x**2 + y**2
            d2  = 1.0d0 + x**2 + 2.0d0*y**2
            d1x = 4.0d0*x
            d1y = 2.0d0*y
            d2x = 2.0d0*x
            d2y = 4.0d0*y
            t  = 5.d0*pi/12.d0
            ct = dcos(t)
            st = dsin(t)
            zkxx = d1*ct*ct + d2*st*st
            zkxy = (d1-d2)*ct*st
            zkyy = d1*st*st + d2*ct*ct

c
            sx  = sin(pi*x)
            cx  = cos(pi*x)
            sy  = sin(pi*y)
            cy  = cos(pi*y)

            fx       =     pi*cx*sy
            fy       =     pi*sx*cy
c
            fe=sx*sy



      fgammae=+(zkxx*fx+zkxy*fy) + fe

      return
      end
c***********************************************************************

      real function fgammaw*8(x,y)
c     function fgammaw - returned values of gamma
c     in the right-hand side of Robin boundary conditions of
c     a differential problem
c     West - left - boundary -> i=1
      implicit real*8 (a-h,o-z)

      pi=acos(-1.)

            d1  = 1.0d0 + 2.0d0*x**2 + y**2
            d2  = 1.0d0 + x**2 + 2.0d0*y**2
            d1x = 4.0d0*x
            d1y = 2.0d0*y
            d2x = 2.0d0*x
            d2y = 4.0d0*y
            t  = 5.d0*pi/12.d0
            ct = dcos(t)
            st = dsin(t)
            zkxx = d1*ct*ct + d2*st*st
            zkxy = (d1-d2)*ct*st
            zkyy = d1*st*st + d2*ct*ct
c
            sx  = sin(pi*x)
            cx  = cos(pi*x)
            sy  = sin(pi*y)
            cy  = cos(pi*y)

 
            fx       =     pi*cx*sy
            fy       =     pi*sx*cy
c
            fe=sx*sy


      fgammaw=-(zkxx*fx+zkxy*fy) + fe
      return
      end
c***********************************************************************

      subroutine coeff
c     Computation of coefficients K.. of the finite-difference
c     scheme in cells
      implicit real*8 (a-h,o-z)
c     M,N - number of nodes in the grid 
      common/MN/ M,N

c     X,Y - arrays (0 to M+1 and 0 to N+1), coordinates of grid nodes
c     X and Y for i=0,M+1; j=0,N+1 are fictitious, and are used
c     for making the computational formulas and program code shorter.
      common/xy/ x(0:201,0:201),y(0:201,0:201)

c     ZK.. - arrays of (1 to M and 1 to N), value of the matrix of
c     coefficient K in nodes
      common/k/ zkxx(200,200), zkxy(200,200),zkyy(200,200)

      do i=1,M
      do j=1,N


c     computation of coefficients

      zkxx(i,j)=fkxx(x(i,j),y(i,j))                                     
      zkxy(i,j)=fkxy(x(i,j),y(i,j))
      zkyy(i,j)=fkyy(x(i,j),y(i,j))
      
      end do
      end do

      return
      end
c*********************************************************************

      real function fkxx*8 (x,y)
c     function fkxx(x,y) computes the coefficients
c     KXX of the original differential equation in point (x,y)
      implicit real*8 (a-h,o-z)
      pi=acos(-1.)
      d1 = 1.0d0 + 2.0d0*x**2 + y**2
      d2 = 1.0d0 + x**2 + 2.0d0*y**2
      t = 5.d0*pi/12.d0
      ct=cos(t)
      st=sin(t)
      fkxx=d1*ct*ct + d2*st*st
      return
      end
c*********************************************************************

      real function fkxy*8 (x,y)
c     function fkxy(x,y) computes the coefficients
c     KXX of the original differential equation in point (x,y)
      implicit real*8 (a-h,o-z)
      pi=acos(-1.)
      d1 = 1.0d0 + 2.0d0*x**2 + y**2
      d2 = 1.0d0 + x**2 + 2.0d0*y**2
      t = 5.d0*pi/12.d0
      ct=cos(t)
      st=sin(t)
      fkxy=(d1-d2)*ct*st
      return
      end
c*********************************************************************

      real function fkyy*8 (x,y)
c     function fkxy(x,y) computes the coefficients
c     KXX of the original differential equation in point (x,y)
      implicit real*8 (a-h,o-z)
      pi=acos(-1.)
      d1 = 1.0d0 + 2.0d0*x**2 + y**2
      d2 = 1.0d0 + x**2 + 2.0d0*y**2
      t = 5.d0*pi/12.d0
      ct=cos(t)
      st=sin(t)
      fkyy=d1*st*st + d2*ct*ct
      return
      end
c*********************************************************************

      subroutine right
c     Computation of the right-hand side of the finite-difference equations
c     in nodes
      implicit real*8 (a-h,o-z)

c     M,N - number of nodes in the grid 
      common/MN/ M,N

c     X,Y - arrays (0 to M+1 and 0 to N+1), coordinates of grid nodes
c     X and Y for i=0,M+1; j=0,N+1 are fictitious, and are used
c     for making the computational formulas and code shorter.
      common/xy/ x(0:201,0:201),y(0:201,0:201)

c     F - array (1 to M-1, and 1 to N-1), values for the right-hand
c     side of the differential equation in cells
      common/f/ f(200,200)

c     comuting the right-hand side of the finite-difference equations
c     ff(x,y) - function which computes the value of the right-hand side
c     of the original differential problem

      do i=1,M-1
      do j=1,N-1
      xcij=0.25*(x(i,j)+x(i+1,j)+x(i+1,j+1)+x(i,j+1))
      ycij=0.25*(y(i,j)+y(i+1,j)+y(i+1,j+1)+y(i,j+1))
      f(i,j)=ff(xcij,ycij)
      end do
      end do

      return
      end
c**********************************************************************

      real function ff*8(x,y)
c     function ff(x,y) - computes the value of the right-hand side
c     of the original differential equation in point (x,y)

      implicit real*8 (a-h,o-z)
      pi=acos(-1.)
     
            d1  = 1.0d0 + 2.0d0*x**2 + y**2
            d2  = 1.0d0 + x**2 + 2.0d0*y**2
            d1x = 4.0d0*x
            d1y = 2.0d0*y
            d2x = 2.0d0*x
            d2y = 4.0d0*y
            t  = 5.d0*pi/12.d0
            ct = dcos(t)
            st = dsin(t)

            zkxx = d1*ct*ct + d2*st*st
            zkxy = (d1-d2)*ct*st
            zkyy = d1*st*st + d2*ct*ct
            zkxx_x     = d1x*ct*ct + d2x*st*st
            zkxy_x     = ( d1x - d2x )*ct*st         
            zkxy_y    = ( d1y - d2y )*ct*st         
            zkyy_y     = d1y*st*st + d2y*ct*ct
c
            sx  = sin(pi*x)
            cx  = cos(pi*x)
            sy  = sin(pi*y)
            cy  = cos(pi*y)
            fx       =     pi*cx*sy
            fy       =     pi*sx*cy
            fxx      = -pi*pi*sx*sy
            fxy      =  pi*pi*cx*cy
            fyy      = -pi*pi*sx*sy
c      
            ff = -( zkxx*fxx + 2*zkxy*fxy + zkyy*fyy
     *       + zkxx_x*fx + zkxy_y*fx + zkxy_x*fy + zkyy_y*fy)
      return
      end
c************************************************************************



