      program p2drnc
c     This is a main program for the solution to the Robin boundary
c     value problem for the Poisson equation in 2-D

c -[d(KXX du/dx)/dx+d(KXY du/dy)/dx+d(KXY du/dx)/dy+d(KYY du/dy)/dy=f
c  nx*(KXX du/dx + KXY du/dy) + ny*(KXY du/dx + KYY du/dy) +
c  + alpha*u = gamma for (x,y) on the boundary,
c   where nx,ny - components of outward unit normal to the boundary.
c=================================================================
c     Discretization - SCALAR functions in NODES (HN),
c                    - VECTOR functions in CELLS (HC)
c=================================================================
      implicit real*8 (a-h,o-z)
c======================================================================
c     eps - constant in the condition of finishing the iteration in 
c     the Gauss-Siedel iteration procedure.  The termination condition is
c
c             (S+1)     (S)
c     max( abs( U(i,j) - U(i,j))) < eps
c     i,j
c 
c     where S is the iteration number.
c

      common/eps/ eps
c=====================================================================

c     Set of coordinates of grid points
c     call subroutine mkgrid
      call mkgrid

c     Set boundary conditions
c     call subroutine bcon

      call bcon


c     Computation of coefficient K in cells
c     call subroutine coeff

      call coeff

c     Computation of the right-hand side of the differential equation in nodes
c     call subroutine right

      call right

c     Computation of coefficients and the right-hand side of the
c     finite-difference equations
c     call subroutine CFDS

      call CFDS 

c     Set of constants for the termination of the iteration process
      eps=1.d-9  

c     Solution of finite-difference equations by the GAUSS-SIEDEL method
c     call subroutine siedel

      call siedel

c     Computing the error
c     call subroutine error

      call error

c-------------------------------------------------------------------------
c     Test output
c-------------------------------------------------------------------------
      call output
      stop
      end

c*************************************************************************

      subroutine CFDS
c     This is subroutine CFDS for the computation of coefficents
c     and the right-hand side of the finite-difference scheme for 
c     Robin boundary value problems for the Poisson equation in 2-D

c -[d(KXX du/dx)/dx+d(KXY du/dy)/dx+d(KXY du/dx)/dy+d(KYY du/dy)/dy=f
c  nx*(KXX du/dx + KXY du/dy) + ny*(KXY du/dx + KYY du/dy) +
c  + alpha*u = gamma for (x,y) on the boundary,

c   where nx,ny - components of outward unit normal to the boundary.
c  for 2-D case.
c=================================================================
c     Discretization - SCALAR functions in NODES (HN),
c                    - VECTOR functions in CELLS (HC)
c=================================================================
      implicit real*8(a-h,o-z)

c     M,N - number of nodes in the grid 
      common/MN/ M,N

c     X,Y - arrays (1 to M and 1 to N), coordinates of grid nodes
      common/xy/ x(200,200),y(200,200)

c     ZK.. - arrays of (1 to M-1 and 1 to N-1), value of the matrix of
c     coefficient K in cells
      common/k/ zkxx(200,200), zkxy(200,200),zkyy(200,200)

c     ALPHAS, ALPHAE, ALPHAN, ALPHAW - arrays which correspond
c     to function alpha(x,y) in the left-hand side of the boundary
c     conditions:    
c     ALPHAS, ALPHAN - south and north boundary - arrays (1 to M )
c     ALPHAW, ALPHAE - west and east boundary - arrays ( 1 to N )
      common/alpha/ alphas(200), alphae(200),
     * alphan(200), alphaw(200)

c     GAMMAS, GAMMAE, GAMMAN, GAMMAW - arrays which correspond
c     to function gamma(x,y) in the right-hand side of the
c     boundary conditions:
c     GAMMAS, GAMMAN - south and north boundary - arrays (1 to M )
c     GAMMAW, GAMMAE - west and east boundary - arrays ( 1 to N )
      common/alpha/ gammas(200), gammae(200),
     * gamman(200), gammaw(200)

c     F - array (2 to M-1, and 2 to N-1), values for the right-hand
c     side of the differential equation in nodes
      common/f/ f(200,200)


c     =================================================================
c     CC,CE,CNE,CN,CNW,CW,CSW,CS,CSE, PHI - arrays of coefficients
c                                           and the right-hand side
c     ( 1 to M and 1 to N ) in the finite-difference equations
c
c     C(i,j)*U(i,j)+
c
c     E(i,j)*U(i+1,j) + NE(i,j)*U(i+1,j+1) + N(i,j)*U(i,j+1) +
c
c     NW(i,j)*U(i-1,j+1) + W(i,j)*U(i-1,j) +SW(i,j)*U(i-1,j-1) +
c
c     S(i,j)*U(i,j-1) + SE(i,j)*U(i+1,j-1) = PHI(i,j)
c


c----------------------  STENCIL --------------------------------
c
c
c
c              NW(i,j)       N(i,j)          NE(i,j)
c             *-------------*---------------*
c             !             !               !
c             !             !               !
c             !             !               !
c             !             !               !
c             !             !               !
c             !             !               !
c             !             !               !
c             !             !               !
c             !             !               !
c             !             !               !
c             !W(i,j)       !C(i,j)         !E(i,j)
c             *-------------*---------------*
c             !             !               !
c             !             !               !
c             !             !               !
c             !             !               !
c             !             !               !
c             !             !               !
c             !             !               !
c             !             !               !
c             !             !               !
c             !             !               !
c             !SW(i,j)      !S(i,j)         !SE(i,j)
c             *-------------*---------------*
c
c
c
c
c
c----------------------  STENCIL --------------------------------
c             FOR THE INTERNAL NODES I=2,...,M-1 ; J=2,...,N-1
c
c
      common/coefds/ cc(200,200),ce(200,200),cne(200,200),
     *               cn(200,200),cnw(200,200),cw(200,200),
     *               csw(200,200),cs(200,200),
     *               cse(200,200),phi(200,200)
c=================================================================

c     VC, VN - arrays for the volume of cells and nodes
      dimension VC(200,200), VN(200,200)

c     ZLS, ZLE, ZLN, ZLW - arrays for the length of nodes
c                          on the boundaries
c     ZLS - south, ZLE - east, ZLN - north, ZLW - west.
      dimension zls(200), zle(200), zln(200), zlw(200)

c     Computing VC and VN

      do  i=1,M-1
      do  j=1,N-1

      VC(i,j) =0.5*((x(i,j)-x(i+1,j+1))*(y(i+1,j)-y(i,j+1)) -
     *              (x(i+1,j)-x(i,j+1))*(y(i,j)-y(i+1,j+1)))

      end do
      end do

      do  i=2,M-1
      do  j=2,N-1
c     Two different choices for VN(i,j)
c     First - average of volumes of adjacent cells
c     VN(i,j)=0.25*(VC(i,j) + VC(i-1,j) + VC(i-1,j-1) + VC(i,j-1))
c     Second "DIAMOND" volume
      VN(i,j)=0.25*((x(i-1,j)-x(i+1,j))*(y(i,j-1)-y(i,j+1)) -
     *              (x(i,j-1)-x(i,j+1))*(y(i-1,j)-y(i+1,j)))
c
      end do
      end do

c     Computing length of nodes zls,zle,zln,zlw

c    South and North boundary - internal nodes
      do i=2,M-1
      zls(i) = 0.5*sqrt((x(i+1,1)-x(i-1,1))**2 +
     *              (y(i+1,1)-y(i-1,1))**2)

      zln(i) = 0.5*sqrt((x(i+1,N)-x(i-1,N))**2 +
     *              (y(i+1,N)-y(i-1,N))**2)
      end do

c     East and West boundary - internal nodes
      do j=2,N-1
      zle(j) = 0.5*sqrt((x(M,j+1)-x(M,j-1))**2 +
     *              (y(M,j+1)-y(M,j-1))**2)

      zlw(j) = 0.5*sqrt((x(1,j+1)-x(1,j-1))**2 +
     *              (y(1,j+1)-y(1,j-1))**2)
      end do

c     Corner (1,1) 
      zl11 = 0.5*sqrt((x(1,2)-x(2,1))**2 +
     *            (y(1,2)-y(2,1))**2)

c     Corner (M,1)
      zlM1 = 0.5*sqrt((x(M-1,1)-x(M,2))**2 + 
     *            (y(M-1,1)-y(M,2))**2)

c     Corner (M,N)
      zlMN = 0.5*sqrt((x(M,N-1)-x(M-1,N))**2 +
     *            (y(M,N-1)-y(M-1,N))**2)

c     Corner (1,N)
      zl1N = 0.5*sqrt((x(2,N)-x(1,N-1))**2 +
     *            (y(2,N)-y(1,N-1))**2) 
c     ================================================================	 

c     computing the coefficients of the finite-difference scheme
c     and the right-hand side in internal nodes i=2,...,M-1 ; j=2,...,N-1
c     ===============================================================


      do  i=2,M-1
      do  j=2,N-1
c
c     COEFFICIENT E(I,J), East, NODE (I+1,J)
c
c    group related to the approximation of term d(zkxx du/dx)/dx
c
         cexx =
     *   + (y(i,j+1)-y(i+1,j))
     *   *(ZKXX(i,j)/(2.d0*VC(i,j)))*(y(i,j)-y(i+1,j+1))
     *   +(y(i+1,j)-y(i,j-1))
     *   *(ZKXX(i,j-1)/(2.d0*VC(i,j-1)))*(y(i+1,j-1)-y(i,j))
c
c    group related to the approximation of term d(zkxy du/dy)/dx
c
         cexy =
     *   -(y(i,j+1)-y(i+1,j))
     *   *(ZKXY(i,j)/(2.d0*VC(i,j)))*(x(i,j)-x(i+1,j+1))
     *   -(y(i+1,j)-y(i,j-1))
     *   *(ZKXY(i,j-1)/(2.d0*VC(i,j-1)))*(x(i+1,j-1)-x(i,j))
c
c    group related to the approximation of term d(zkxy du/dx)/dy
c
         ceyx =
     *   -(x(i,j+1)-x(i+1,j))
     *   *(ZKXY(i,j)/(2.d0*VC(i,j)))*(y(i,j)-y(i+1,j+1))
     *   -(x(i+1,j)-x(i,j-1))
     *   *(ZKXY(i,j-1)/(2.d0*VC(i,j-1)))*(y(i+1,j-1)-y(i,j))
c
c    group related to the approximation of term d(zkyy du/dy)/dy
c
         ceyy =
     *   +(x(i,j+1)-x(i+1,j))
     *   *(ZKYY(i,j)/(2.d0*VC(i,j)))*(x(i,j)-x(i+1,j+1))
     *   +(x(i+1,j)-x(i,j-1))
     *   *(ZKYY(i,j-1)/(2.d0*VC(i,j-1)))*(x(i+1,j-1)-x(i,j))

      ce(i,j) = cexx + cexy + ceyx + ceyy

c
c     COEFFICIENT NE(I,J), North-East, NODE (I+1,J+1)
c
c
c
c    group related to the approximation of term d(zkxx du/dx)/dx
c
      cnexx = + (y(i,j+1)-y(i+1,j))
     *        *(ZKXX(i,j)/(2.d0*VC(i,j)))*(y(i+1,j)-y(i,j+1))
c
c    group related to approximation of term d(zkxy du/dy)/dx
c
      cnexy = -(y(i,j+1)-y(i+1,j))
     *        *(ZKXY(i,j)/(2.d0*VC(i,j)))*(x(i+1,j)-x(i,j+1))


c
c    group related to the approximation of term d(zkxy du/dx)/dy
c
      cneyx = -(x(i,j+1)-x(i+1,j))
     *        *(ZKXY(i,j)/(2.d0*VC(i,j)))*(y(i+1,j)-y(i,j+1))


c
c    group related to the approximation of term d(zkyy du/dy)/dy
c
      cneyy = + (x(i,j+1)-x(i+1,j))
     *        *(ZKYY(i,j)/(2.d0*VC(i,j)))*(x(i+1,j)-x(i,j+1))



      cne(i,j) = cnexx + cnexy + cneyx + cneyy

c
c     COEFFICIENT N(I,J), North,NODE (I,J+1)
c
c    group related to the approximation of term d(zkxx du/dx)/dx
c
         cnxx =
     *   -(y(i,j+1)-y(i+1,j))
     *   *(ZKXX(i,j)/(2.d0*VC(i,j)))*(y(i,j)-y(i+1,j+1))
     *   +(y(i-1,j)-y(i,j+1))
     *   *(ZKXX(i-1,j)/(2.d0*VC(i-1,j)))*(y(i,j)-y(i-1,j+1))
c
c    group related to the approximation of term d(zkxy du/dy)/dx
c
         cnxy =
     *   +(y(i,j+1)-y(i+1,j))
     *   *(ZKXY(i,j)/(2.d0*VC(i,j)))*(x(i,j)-x(i+1,j+1))
     *   -(y(i-1,j)-y(i,j+1))
     *   *(ZKXY(i-1,j)/(2.d0*VC(i-1,j)))*(x(i,j)-x(i-1,j+1))
c
c    group related to the approximation of term d(zkxy du/dx)/dy
c
         cnyx =
     *   +(x(i,j+1)-x(i+1,j))
     *   *(ZKXY(i,j)/(2.d0*VC(i,j)))*(y(i,j)-y(i+1,j+1))
     *   -(x(i-1,j)-x(i,j+1))
     *   *(ZKXY(i-1,j)/(2.d0*VC(i-1,j)))*(y(i,j)-y(i-1,j+1))
c
c    group related to the approximation of term d(zkyy du/dy)/dy
c
         cnyy =
     *   -(x(i,j+1)-x(i+1,j))
     *   *(ZKYY(i,j)/(2.d0*VC(i,j)))*(x(i,j)-x(i+1,j+1))
     *   +(x(i-1,j)-x(i,j+1))
     *   *(ZKYY(i-1,j)/(2.d0*VC(i-1,j)))*(x(i,j)-x(i-1,j+1))

      cn(i,j) = cnxx + cnxy + cnyx + cnyy


c
c     COEFFICIENT NW(I,J), North-West, NODE (I-1,J+1)
c
c
c
c    group related to the approximation of term d(zkxx du/dx)/dx
c
      cnwxx =
     *        -(y(i-1,j)-y(i,j+1))
     *        *(ZKXX(i-1,j)/(2.d0*VC(i-1,j)))*(y(i-1,j)-y(i,j+1))

c
c    group related to the approximation of term d(zkxy du/dy)/dx
c
      cnwxy =
     *        +(y(i-1,j)-y(i,j+1))
     *        *(ZKXY(i-1,j)/(2.d0*VC(i-1,j)))*(x(i-1,j)-x(i,j+1))


c
c    group related to the approximation of term d(zkxy du/dx)/dy
c
      cnwyx =
     *        +(x(i-1,j)-x(i,j+1))
     *        *(ZKXY(i-1,j)/(2.d0*VC(i-1,j)))*(y(i-1,j)-y(i,j+1))


c
c    group related to the approximation of term d(zkyy du/dy)/dy
c
      cnwyy =
     *        -(x(i-1,j)-x(i,j+1))
     *        *(ZKYY(i-1,j)/(2.d0*VC(i-1,j)))*(x(i-1,j)-x(i,j+1))


      cnw(i,j) = cnwxx + cnwxy + cnwyx + cnwyy


c
c     COEFFICIENT W(I,J), West, NODE (I-1,J)
c
c    group related to the approximation of term d(zkxx du/dx)/dx
c
         cwxx =
     *   -(y(i-1,j)-y(i,j+1))
     *   *(ZKXX(i-1,j)/(2.d0*VC(i-1,j)))*(y(i,j)-y(i-1,j+1))
     *   -(y(i,j-1)-y(i-1,j))
     *   *(ZKXX(i-1,j-1)/(2.d0*VC(i-1,j-1)))*(y(i-1,j-1)-y(i,j))
c
c    group related to the approximation of term d(zkxy du/dy)/dx
c
         cwxy =
     *   + (y(i-1,j)-y(i,j+1))
     *   *(ZKXY(i-1,j)/(2.d0*VC(i-1,j)))*(x(i,j)-x(i-1,j+1))
     *   + (y(i,j-1)-y(i-1,j))
     *   *(ZKXY(i-1,j-1)/(2.d0*VC(i-1,j-1)))*(x(i-1,j-1)-x(i,j))
c
c    group related to the approximation of term d(zkxy du/dx)/dy
c
         cwyx =
     *   +(x(i-1,j)-x(i,j+1))
     *   *(ZKXY(i-1,j)/(2.d0*VC(i-1,j)))*(y(i,j)-y(i-1,j+1))
     *   +(x(i,j-1)-x(i-1,j))
     *   *(ZKXY(i-1,j-1)/(2.d0*VC(i-1,j-1)))*(y(i-1,j-1)-y(i,j))
c
c    group related to the approximation of term d(zkyy du/dy)/dy
c
         cwyy =
     *   -(x(i-1,j)-x(i,j+1))
     *   *(ZKYY(i-1,j)/(2.d0*VC(i-1,j)))*(x(i,j)-x(i-1,j+1))
     *   -(x(i,j-1)-x(i-1,j))
     *   *(ZKYY(i-1,j-1)/(2.d0*VC(i-1,j-1)))*(x(i-1,j-1)-x(i,j))

      cw(i,j) = cwxx + cwxy + cwyx + cwyy

c
c     COEFFICIENT SW(I,J), Sowth-West, NODE (I-1,J-1)
c
c
c
c    group related to the approximation of term d(zkxx du/dx)/dx
c
      cswxx =
     *        -(y(i,j-1)-y(i-1,j))
     *        *(ZKXX(i-1,j-1)/(2.d0*VC(i-1,j-1)))*(y(i,j-1)-y(i-1,j))

c
c    group related to the approximation of term d(zkxy du/dy)/dx
c
      cswxy =
     *        +(y(i,j-1)-y(i-1,j))
     *        *(ZKXY(i-1,j-1)/(2.d0*VC(i-1,j-1)))*(x(i,j-1)-x(i-1,j))


c
c    group related to the approximation of term d(zkxy du/dx)/dy
c
      cswyx =
     *        +(x(i,j-1)-x(i-1,j))
     *        *(ZKXY(i-1,j-1)/(2.d0*VC(i-1,j-1)))*(y(i,j-1)-y(i-1,j))


c
c    group related to the approximation of term d(zkyy du/dy)/dy
c
      cswyy =
     *        -(x(i,j-1)-x(i-1,j))
     *        *(ZKYY(i-1,j-1)/(2.d0*VC(i-1,j-1)))*(x(i,j-1)-x(i-1,j))


      csw(i,j) = cswxx + cswxy + cswyx + cswyy



c
c     COEFFICIENT S(I,J), South, NODE (I,J-1)
c
c    group related to the approximation of term d(zkxx du/dx)/dx
c
         csxx =
     *   +(y(i,j-1)-y(i-1,j))
     *   *(ZKXX(i-1,j-1)/(2.d0*VC(i-1,j-1)))*(y(i-1,j-1)-y(i,j))
     *   -(y(i+1,j)-y(i,j-1))
     *   *(ZKXX(i,j-1)/(2.d0*VC(i,j-1)))*(y(i+1,j-1)-y(i,j))
c
c    group related to the approximation of term d(zkxy du/dy)/dx
c
         csxy =
     *   -(y(i,j-1)-y(i-1,j))
     *   *(ZKXY(i-1,j-1)/(2.d0*VC(i-1,j-1)))*(x(i-1,j-1)-x(i,j))
     *   +(y(i+1,j)-y(i,j-1))
     *   *(ZKXY(i,j-1)/(2.d0*VC(i,j-1)))*(x(i+1,j-1)-x(i,j))
c
c    group related to the approximation of term d(zkxy du/dx)/dy
c
         csyx =
     *   -(x(i,j-1)-x(i-1,j))
     *   *(ZKXY(i-1,j-1)/(2.d0*VC(i-1,j-1)))*(y(i-1,j-1)-y(i,j))
     *   +(x(i+1,j)-x(i,j-1))
     *   *(ZKXY(i,j-1)/(2.d0*VC(i,j-1)))*(y(i+1,j-1)-y(i,j))
c
c    group related to the approximation of term d(zkyy du/dy)/dy
c
         csyy =
     *   +(x(i,j-1)-x(i-1,j))
     *   *(ZKYY(i-1,j-1)/(2.d0*VC(i-1,j-1)))*(x(i-1,j-1)-x(i,j))
     *   -(x(i+1,j)-x(i,j-1))
     *   *(ZKYY(i,j-1)/(2.d0*VC(i,j-1)))*(x(i+1,j-1)-x(i,j))

      cs(i,j) = csxx + csxy + csyx + csyy



c
c     COEFFICIENT SE(I,J), South-East, NODE (I+1,J-1)
c
c
c
c    group related to the approximation of term d(zkxx du/dx)/dx
c
      csexx =
     *        + (y(i+1,j)-y(i,j-1))
     *        *(ZKXX(i,j-1)/(2.d0*VC(i,j-1)))*(y(i,j-1)-y(i+1,j))

c
c    group related to the approximation of term d(zkxy du/dy)/dx
c
      csexy =
     *        -(y(i+1,j)-y(i,j-1))
     *        *(ZKXY(i,j-1)/(2.d0*VC(i,j-1)))*(x(i,j-1)-x(i+1,j))


c
c    group related to the approximation of term d(zkxy du/dx)/dy
c
      cseyx =
     *        -(x(i+1,j)-x(i,j-1))
     *        *(ZKXY(i,j-1)/(2.d0*VC(i,j-1)))*(y(i,j-1)-y(i+1,j))


c
c    group related to the approximation of term d(zkyy du/dy)/dy
c
      cseyy =
     *        + (x(i+1,j)-x(i,j-1))
     *        *(ZKYY(i,j-1)/(2.d0*VC(i,j-1)))*(x(i,j-1)-x(i+1,j))


      cse(i,j) = csexx + csexy + cseyx + cseyy
c
c      COEFFICIENT IN NODE (I,J) - CC(i,j)


      cc(i,j) = - ( ce(i,j) + cne(i,j) + cn(i,j) + cnw(i,j) +
     *          cw(i,j) + csw(i,j) + cs(i,j) + cse(i,j))


      phi(i,j)=f(i,j) * ( 2.0d0 * VN(i,j) )

      end do
      end do

c     computing the coefficients of the finite-difference scheme
c     and the right-hand side on the south boundary (not corner nodes)
c     i=2,...,M-1 ; j=1
c     ===============================================================

c----------------------  STENCIL --------------------------------
c
c
c
c              NW(i,1)       N(i,1)          NE(i,1)
c             *-------------*---------------*
c             !             !               !
c             !             !               !
c             !             !               !
c             !             !               !
c             !             !               !
c             !             !               !
c             !             !               !
c             !             !               !
c             !             !               !
c             !             !               !
c             !W(i,1)       !C(i,1)         !E(i,1)
c             *-------------*---------------*
c
c----------------------  STENCIL --------------------------------
c             FOR THE INTERNAL NODES I=2,...,M-1 ; J=1

      do i=2,M-1

c     coefficient ce(i,1)

c     group related to the approximation of term  
c     nx*(kxx du/dx + kxy du/dy)
      cex = -0.5*(y(i,2)-y(i+1,1))*
     * ( zkxx(i,1)*(y(i+1,2)-y(i,1))/(2.*VC(i,1))-
     * zkxy(i,1)*(x(i+1,2)-x(i,1))/(2.*VC(i,1)))

c     group related to the approximation of term  
c     ny*(kxy du/dx + kyy du/dy)
      cey = +0.5*(x(i,2)-x(i+1,1))*
     * ( zkxy(i,1)*(y(i+1,2)-y(i,1))/(2.*VC(i,1))-
     * zkyy(i,1)*(x(i+1,2)-x(i,1))/(2.*VC(i,1)))


      ce(i,1) = cex + cey

c     coefficient cne(i,1)

c     group related to the approximation of term  
c     nx*(kxx du/dx + kxy du/dy)
      cnex = -0.5*(y(i,2)-y(i+1,1))*
     * ( zkxx(i,1)*(y(i,2)-y(i+1,1))/(2.*VC(i,1))-
     * zkxy(i,1)*(x(i,2)-x(i+1,1))/(2.*VC(i,1)))
      

c     group related to the approximation of term  
c     ny*(kxy du/dx + kyy du/dy)
      cney = +0.5*(x(i,2)-x(i+1,1))*
     * ( zkxy(i,1)*(y(i,2)-y(i+1,1))/(2.*VC(i,1))-
     * zkyy(i,1)*(x(i,2)-x(i+1,1))/(2.*VC(i,1)))

      cne(i,1) = cnex + cney

c     coefficient cn(i,1)

c     group related to the approximation of term  
c     nx*(kxx du/dx + kxy du/dy)
      cnx = -0.5*(y(i,2)-y(i+1,1))*
     * (- zkxx(i,1)*(y(i+1,2)-y(i,1))/(2.*VC(i,1))+
     * zkxy(i,1)*(x(i+1,2)-x(i,1))/(2.*VC(i,1))) 
     *     + 0.5*(y(i,2)-y(i-1,1))*
     * (zkxx(i-1,1)*(y(i-1,2)-y(i,1))/(2.*VC(i-1,1))
     * -zkxy(i-1,1)*(x(i-1,2)-x(i,1))/(2.*VC(i-1,1)))

c     group related to the approximation of term  
c     ny*(kxy du/dx + kyy du/dy)
      cny = +0.5*(x(i,2)-x(i+1,1))*
     * ( -zkxy(i,1)*(y(i+1,2)-y(i,1))/(2.*VC(i,1))+
     * zkyy(i,1)*(x(i+1,2)-x(i,1))/(2.*VC(i,1)))
     *     - 0.5*(x(i,2)-x(i-1,1))*
     * (zkxy(i-1,1)*(y(i-1,2)-y(i,1))/(2.*VC(i-1,1))
     * -zkyy(i-1,1)*(x(i-1,2)-x(i,1))/(2.*VC(i-1,1)))

      cn(i,1) = cnx + cny

c     coefficient cnw

c     group related to the approximation of term  
c     nx*(kxx du/dx + kxy du/dy)
       
      cnwx = +0.5*(y(i,2)-y(i-1,1))*
     * (-zkxx(i-1,1)*(y(i,2)-y(i-1,1))/(2.*VC(i-1,1))+
     *   zkxy(i-1,1)*(x(i,2)-x(i-1,1))/(2.*VC(i-1,1)))

c     group related to the approximation of term  
c     ny*(kxy du/dx + kyy du/dy)

      cnwy = -0.5*(x(i,2)-x(i-1,1))*
     * (-zkxy(i-1,1)*(y(i,2)-y(i-1,1))/(2.*VC(i-1,1))+
     *   zkyy(i-1,1)*(x(i,2)-x(i-1,1))/(2.*VC(i-1,1)))

      cnw(i,1) = cnwx + cnwy

c     coefficient cw

c     group related to the approximation of term  
c     nx*(kxx du/dx + kxy du/dy)

      cwx = +0.5*(y(i,2)-y(i-1,1))*
     *  (-zkxx(i-1,1)*(y(i-1,2)-y(i,1))/(2.*VC(i-1,1))+
     *    zkxy(i-1,1)*(x(i-1,2)-x(i,1))/(2.*VC(i-1,1)))

c     group related to the approximation of term  
c     ny*(kxy du/dx + kyy du/dy)

      cwy = -0.5*(x(i,2)-x(i-1,1))*
     *  (-zkxy(i-1,1)*(y(i-1,2)-y(i,1))/(2.*VC(i-1,1))+
     *    zkyy(i-1,1)*(x(i-1,2)-x(i,1))/(2.*VC(i-1,1)))

      cw(i,1) = cwx + cwy

c     coefficient cc

      cc(i,1) = -(ce(i,1) + cne(i,1) + cn(i,1) +
     *          cnw(i,1) + cw(i,1)) +
     *                    alphas(i)*zls(i)

c      right-hand side
      phi(i,1) = gammas(i)*zls(i)
      end do

c       EAST boundary - internal nodes (i=M, j=2,...,N-1)

c----------------------  STENCIL --------------------------------
c
c
c
c              NW(M,j)       N(M,j)          
c             *-------------*
c             !             !               
c             !             !               
c             !             !              
c             !             !               
c             !             !               
c             !             !               
c             !             !               
c             !             !               
c             !             !               
c             !             !               
c             !W(M,j)       !C(M,j)         
c             *-------------*
c             !             !               
c             !             !               
c             !             !               
c             !             !               
c             !             !               
c             !             !               
c             !             !               
c             !             !               
c             !             !               
c             !             !               
c             !SW(M,j)      !S(M,j)        
c             *-------------*
c
c
c
c
c
c----------------------  STENCIL --------------------------------
c             FOR THE INTERNAL NODES I=M ; J=2,...,N-1
c
c
        do j=2,N-1
c     Coefficient cn

c     group related to the approximation of term  
c     nx*(kxx du/dx + kxy du/dy)

      cnx = +0.5*(y(M,j+1)-y(M-1,j))*
     * (zkxx(M-1,j)*(y(M-1,j+1)-y(M,j))/(2*VC(M-1,j))-
     *  zkxy(M-1,j)*(x(M-1,j+1)-x(M,j))/(2*VC(M-1,j)))

      
c     group related to the approximation of term  
c     ny*(kxy du/dx + kyy du/dy)

      cny = -0.5*(x(M,j+1)-x(M-1,j))*
     * (zkxy(M-1,j)*(y(M-1,j+1)-y(M,j))/(2*VC(M-1,j))-
     *  zkyy(M-1,j)*(x(M-1,j+1)-x(M,j))/(2*VC(M-1,j)))

      cn(M,j) = cnx + cny

c     Coefficient cnw

c     group related to the approximation of term  
c     nx*(kxx du/dx + kxy du/dy)
      
      cnwx = +0.5*(y(M,j+1)-y(M-1,j))*
     *  (-zkxx(M-1,j)*(y(M,j+1)-y(M-1,j))/(2*VC(M-1,j))+
     *    zkxy(M-1,j)*(x(M,j+1)-x(M-1,j))/(2*VC(M-1,j)))


c     group related to the approximation of term  
c     ny*(kxy du/dx + kyy du/dy)
      cnwy = -0.5*(x(M,j+1)-x(M-1,j))*
     *  (-zkxy(M-1,j)*(y(M,j+1)-y(M-1,j))/(2*VC(M-1,j))+
     *    zkyy(M-1,j)*(x(M,j+1)-x(M-1,j))/(2*VC(M-1,j)))

      cnw(M,j) = cnwx + cnwy

c     Coefficient cw

c     group related to the approximation of term  
c     nx*(kxx du/dx + kxy du/dy)

      cwx = +0.5*(y(M-1,j)-y(M,j-1))*
     *  (-zkxx(M-1,j-1)*(y(M,j)-y(M-1,j-1))/(2.*VC(M-1,j-1))+
     *    zkxy(M-1,j-1)*(x(M,j)-x(M-1,j-1))/(2.*VC(M-1,j-1)))
     *      +0.5*(y(M,j+1)-y(M-1,j))*
     *  (-zkxx(M-1,j)*(y(M-1,j+1)-y(M,j))/(2.*VC(M-1,j))+
     *    zkxy(M-1,j)*(x(M-1,j+1)-x(M,j))/(2.*VC(M-1,j)))

c     group related to the approximation of term  
c     ny*(kxy du/dx + kyy du/dy)

      cwy = -0.5*(x(M-1,j)-x(M,j-1))*
     *  (-zkxy(M-1,j-1)*(y(M,j)-y(M-1,j-1))/(2.*VC(M-1,j-1))+
     *    zkyy(M-1,j-1)*(x(M,j)-x(M-1,j-1))/(2.*VC(M-1,j-1)))
     *      -0.5*(x(M,j+1)-x(M-1,j))*
     *  (-zkxy(M-1,j)*(y(M-1,j+1)-y(M,j))/(2.*VC(M-1,j))+
     *    zkyy(M-1,j)*(x(M-1,j+1)-x(M,j))/(2.*VC(M-1,j)))

      cw(M,j) = cwx + cwy

c     Coefficient csw

c     group related to the approximation of term  
c     nx*(kxx du/dx + kxy du/dy)

      cswx = -0.5*(y(M-1,j)-y(M,j-1))*
     *  (zkxx(M-1,j-1)*(y(M-1,j)-y(M,j-1))/(2.*VC(M-1,j-1))-
     *   zkxy(M-1,j-1)*(x(M-1,j)-x(M,j-1))/(2.*VC(M-1,j-1)))
     

c     group related to the approximation of term  
c     ny*(kxy du/dx + kyy du/dy)

      cswy = +0.5*(x(M-1,j)-x(M,j-1))*
     *  (zkxy(M-1,j-1)*(y(M-1,j)-y(M,j-1))/(2.*VC(M-1,j-1))-
     *   zkyy(M-1,j-1)*(x(M-1,j)-x(M,j-1))/(2.*VC(M-1,j-1)))

      csw(M,j) = cswx + cswy

c     Coefficient cs

c     group related to the approximation of term  
c     nx*(kxx du/dx + kxy du/dy)

      csx = +0.5*(y(M-1,j)-y(M,j-1))*
     *  (zkxx(M-1,j-1)*(y(M,j)-y(M-1,j-1))/(2.*VC(M-1,j-1))-
     *   zkxy(M-1,j-1)*(x(M,j)-x(M-1,j-1))/(2.*VC(M-1,j-1)))
     

c     group related to the approximation of term  
c     ny*(kxy du/dx + kyy du/dy)

      csy = -0.5*(x(M-1,j)-x(M,j-1))*
     *  (zkxy(M-1,j-1)*(y(M,j)-y(M-1,j-1))/(2.*VC(M-1,j-1))-
     *   zkyy(M-1,j-1)*(x(M,j)-x(M-1,j-1))/(2.*VC(M-1,j-1)))

      cs(M,j) = csx + csy

c    Coefficient cc
     
      cc(M,j) = -(cn(M,j) + cnw(M,j) + cw(M,j) +
     *         csw(M,j) + cs(M,j)) +
     *                    alphae(j)*zle(j)

c      right-hand side
      phi(M,j) = gammae(j)*zle(j)
      end do

c      Coefficients on the North boundary - internal nodes
c          i=2,...,M-1 ; j=N

c----------------------  STENCIL --------------------------------
c
c
c              W(i,N)        C(i,N)          E(i,N)
c             *-------------*---------------*
c             !             !               !
c             !             !               !
c             !             !               !
c             !             !               !
c             !             !               !
c             !             !               !
c             !             !               !
c             !             !               !
c             !             !               !
c             !SW(i,N)      !S(i,N)         !SE(i,N)
c             *-------------*---------------*
c
c
c
c
c
c-----------------  STENCIL  i=2,..,M-1 ; j=N --------------

c
c
       do i = 2,M-1
           
c      Coefficient cw

c     group related to the approximation of term  
c     nx*(kxx du/dx + kxy du/dy)

       cwx = +0.5*(y(i-1,N)-y(i,N-1))*
     *  (-zkxx(i-1,N-1)*(y(i,N)-y(i-1,N-1))/(2.*VC(i-1,N-1))+
     *    zkxy(i-1,N-1)*(x(i,N)-x(i-1,N-1))/(2.*VC(i-1,N-1)))

c     group related to the approximation of term  
c     ny*(kxy du/dx + kyy du/dy)

       cwy = -0.5*(x(i-1,N)-x(i,N-1))*
     *  (-zkxy(i-1,N-1)*(y(i,N)-y(i-1,N-1))/(2.*VC(i-1,N-1))+
     *    zkyy(i-1,N-1)*(x(i,N)-x(i-1,N-1))/(2.*VC(i-1,N-1)))

       cw(i,N) = cwx + cwy
 
c      Coefficient csw

c     group related to the approximation of term  
c     nx*(kxx du/dx + kxy du/dy)

       cswx = +0.5*(y(i-1,N)-y(i,N-1))*
     *  (-zkxx(i-1,N-1)*(y(i-1,N)-y(i,N-1))/(2.*VC(i-1,N-1))+
     *    zkxy(i-1,N-1)*(x(i-1,N)-x(i,N-1))/(2.*VC(i-1,N-1)))

c     group related to the approximation of term  
c     ny*(kxy du/dx + kyy du/dy)

       cswy = -0.5*(x(i-1,N)-x(i,N-1))*
     *  (-zkxy(i-1,N-1)*(y(i-1,N)-y(i,N-1))/(2.*VC(i-1,N-1))+
     *    zkyy(i-1,N-1)*(x(i-1,N)-x(i,N-1))/(2.*VC(i-1,N-1)))

       csw(i,N) = cswx + cswy
 
c     Coefficient cs

c     group related to the approximation of term  
c     nx*(kxx du/dx + kxy du/dy)

       csx = -0.5*(y(i-1,N)-y(i,N-1))*
     *  (-zkxx(i-1,N-1)*(y(i,N)-y(i-1,N-1))/(2.*VC(i-1,N-1))+
     *    zkxy(i-1,N-1)*(x(i,N)-x(i-1,N-1))/(2.*VC(i-1,N-1)))
     *       +0.5*(y(i+1,N)-y(i,N-1))*
     *  (zkxx(i,N-1)*(y(i,N)-y(i+1,N-1))/(2.*VC(i,N-1))-
     *   zkxy(i,N-1)*(x(i,N)-x(i+1,N-1))/(2.*VC(i,N-1)))

c     group related to the approximation of term  
c     ny*(kxy du/dx + kyy du/dy)

       csy = +0.5*(x(i-1,N)-x(i,N-1))*
     *  (-zkxy(i-1,N-1)*(y(i,N)-y(i-1,N-1))/(2.*VC(i-1,N-1))+
     *    zkyy(i-1,N-1)*(x(i,N)-x(i-1,N-1))/(2.*VC(i-1,N-1)))
     *       -0.5*(x(i+1,N)-x(i,N-1))*
     *  (zkxy(i,N-1)*(y(i,N)-y(i+1,N-1))/(2.*VC(i,N-1))-
     *   zkyy(i,N-1)*(x(i,N)-x(i+1,N-1))/(2.*VC(i,N-1)))

       cs(i,N) = csx + csy
 

c      Coefficient cse

c     group related to the approximation of term  
c     nx*(kxx du/dx + kxy du/dy)

       csex = +0.5*(y(i+1,N)-y(i,N-1))*
     *  (-zkxx(i,N-1)*(y(i+1,N)-y(i,N-1))/(2.*VC(i,N-1))+
     *    zkxy(i,N-1)*(x(i+1,N)-x(i,N-1))/(2.*VC(i,N-1)))

c     group related to the approximation of term  
c     ny*(kxy du/dx + kyy du/dy)

       csey = -0.5*(x(i+1,N)-x(i,N-1))*
     *  (-zkxy(i,N-1)*(y(i+1,N)-y(i,N-1))/(2.*VC(i,N-1))+
     *    zkyy(i,N-1)*(x(i+1,N)-x(i,N-1))/(2.*VC(i,N-1)))
       
       cse(i,N) = csex + csey

c     Coefficient ce

c     group related to the approximation of term  
c     nx*(kxx du/dx + kxy du/dy)

       cex =  -0.5*(y(i+1,N)-y(i,N-1))*
     *  (zkxx(i,N-1)*(y(i,N)-y(i+1,N-1))/(2.*VC(i,N-1))-
     *   zkxy(i,N-1)*(x(i,N)-x(i+1,N-1))/(2.*VC(i,N-1)))

c     group related to the approximation of term  
c     ny*(kxy du/dx + kyy du/dy)

       cey =  +0.5*(x(i+1,N)-x(i,N-1))*
     *  (zkxy(i,N-1)*(y(i,N)-y(i+1,N-1))/(2.*VC(i,N-1))-
     *   zkyy(i,N-1)*(x(i,N)-x(i+1,N-1))/(2.*VC(i,N-1)))

       ce(i,N) = cex + cey

c      Coefficient cc
  
       cc(i,N) = - (cw(i,N) + csw(i,N) +
     *            cs(i,N) + cse(i,N) +ce(i,N))+
     *                    alphan(i)*zln(i)

c      right-hand side
      phi(i,N) = gamman(i)*zln(i)
 
      end do

c      Computing coefficients on the West boundary 
c                i = 1 ; j = 2,...,M-1
 
c----------------------  STENCIL --------------------------------
c
c
c
c                            N(1,j)          NE(1,j)
c                           *---------------*
c                           !               !
c                           !               !
c                           !               !
c                           !               !
c                           !               !
c                           !               !
c                           !               !
c                           !               !
c                           !               !
c                           !               !
c                           !C(1,j)         !E(1,j)
c                           *---------------*
c                           !               !
c                           !               !
c                           !               !
c                           !               !
c                           !               !
c                           !               !
c                           !               !
c                           !               !
c                           !               !
c                           !               !
c                           !S(1,j)         !SE(1,j)
c                           *---------------*
c
c
       do j = 2,N-1
 
c     Coefficient ce
    
c     group related to the approximation of term  
c     nx*(kxx du/dx + kxy du/dy)

       cex = -0.5*(y(1,j+1)-y(2,j))*
     *   (zkxx(1,j)*(y(2,j+1)-y(1,j))/(2.*VC(1,j))-
     *    zkxy(1,j)*(x(2,j+1)-x(1,j))/(2.*VC(1,j)))
     *       -0.5*(y(2,j)-y(1,j-1))*
     *    (zkxx(1,j-1)*(y(1,j)-y(2,j-1))/(2.*VC(1,j-1))-
     *     zkxy(1,j-1)*(x(1,j)-x(2,j-1))/(2.*VC(1,j-1)))
     


c     group related to the approximation of term  
c     ny*(kxy du/dx + kyy du/dy)
       cey = +0.5*(x(1,j+1)-x(2,j))*
     *   (zkxy(1,j)*(y(2,j+1)-y(1,j))/(2.*VC(1,j))-
     *    zkyy(1,j)*(x(2,j+1)-x(1,j))/(2.*VC(1,j)))
     *       +0.5*(x(2,j)-x(1,j-1))*
     *    (zkxy(1,j-1)*(y(1,j)-y(2,j-1))/(2.*VC(1,j-1))-
     *     zkyy(1,j-1)*(x(1,j)-x(2,j-1))/(2.*VC(1,j-1)))

        ce(1,j) = cex + cey



c       Coefficient ne

c     group related to the approximation of term  
c     nx*(kxx du/dx + kxy du/dy)
      cnex = -0.5*(y(1,j+1)-y(2,j))*
     *   (zkxx(1,j)*(y(1,j+1)-y(2,j))/(2.*VC(1,j))-
     *    zkxy(1,j)*(x(1,j+1)-x(2,j))/(2.*VC(1,j)))
        

c     group related to the approximation of term  
c     ny*(kxy du/dx + kyy du/dy)
      cney = +0.5*(x(1,j+1)-x(2,j))*
     *   (zkxy(1,j)*(y(1,j+1)-y(2,j))/(2.*VC(1,j))-
     *    zkyy(1,j)*(x(1,j+1)-x(2,j))/(2.*VC(1,j)))

       cne(1,j) = cnex + cney

c     Coefficient cn
    
c     group related to the approximation of term  
c     nx*(kxx du/dx + kxy du/dy)

       cnx = +0.5*(y(1,j+1)-y(2,j))*
     *   (zkxx(1,j)*(y(2,j+1)-y(1,j))/(2.*VC(1,j))-
     *    zkxy(1,j)*(x(2,j+1)-x(1,j))/(2.*VC(1,j)))
     


c     group related to the approximation of term  
c     ny*(kxy du/dx + kyy du/dy)
       cny = -0.5*(x(1,j+1)-x(2,j))*
     *   (zkxy(1,j)*(y(2,j+1)-y(1,j))/(2.*VC(1,j))-
     *    zkyy(1,j)*(x(2,j+1)-x(1,j))/(2.*VC(1,j)))

        cn(1,j) = cnx + cny


c     Coefficient cs
    
c     group related to the approximation of term  
c     nx*(kxx du/dx + kxy du/dy)

       csx = +0.5*(y(2,j)-y(1,j-1))*
     *    (zkxx(1,j-1)*(y(1,j)-y(2,j-1))/(2.*VC(1,j-1))-
     *     zkxy(1,j-1)*(x(1,j)-x(2,j-1))/(2.*VC(1,j-1)))
     


c     group related to the approximation of term  
c     ny*(kxy du/dx + kyy du/dy)
      csy =  -0.5*(x(2,j)-x(1,j-1))*
     *    (zkxy(1,j-1)*(y(1,j)-y(2,j-1))/(2.*VC(1,j-1))-
     *     zkyy(1,j-1)*(x(1,j)-x(2,j-1))/(2.*VC(1,j-1)))

        cs(1,j) = csx + csy

c     Coefficient cse
    
c     group related to the approximation of term  
c     nx*(kxx du/dx + kxy du/dy)

       csex = -0.5*(y(2,j)-y(1,j-1))*
     *    (zkxx(1,j-1)*(y(2,j)-y(1,j-1))/(2.*VC(1,j-1))-
     *     zkxy(1,j-1)*(x(2,j)-x(1,j-1))/(2.*VC(1,j-1)))
     


c     group related to the approximation of term  
c     ny*(kxy du/dx + kyy du/dy)
      csey =  +0.5*(x(2,j)-x(1,j-1))*
     *    (zkxy(1,j-1)*(y(2,j)-y(1,j-1))/(2.*VC(1,j-1))-
     *     zkyy(1,j-1)*(x(2,j)-x(1,j-1))/(2.*VC(1,j-1)))

        cse(1,j) = csex + csey

c     Coefficient cc
        cc(1,j) = -(ce(1,j) + cne(1,j) +
     *          cn(1,j) + cs(1,j) +cse(1,j)) +
     *            alphaw(j)*zlw(j)
        
c     Right-hand side
      phi(1,j) = gammaw(j)*zlw(j)
 
      end do

c     Corner (1,1)
 
c----------------------  STENCIL --------------------------------
c
c
c
c                            N(1,1)          NE(1,1)
c                           *---------------*
c                           !               !
c                           !               !
c                           !               !
c                           !               !
c                           !               !
c                           !               !
c                           !               !
c                           !               !
c                           !               !
c                           !               !
c                           !C(1,1)         !E(1,1)
c                           *---------------*
c
 
c     Coefficient ce

c     group related to the approximation of term  
c     nx*(kxx du/dx + kxy du/dy)
       cex = -0.5*(y(1,2)-y(2,1))*
     * (zkxx(1,1)*(y(2,2)-y(1,1))/(2.*VC(1,1))-
     *  zkxy(1,1)*(x(2,2)-x(1,1))/(2.*VC(1,1)))
      
c     group related to the approximation of term  
c     ny*(kxy du/dx + kyy du/dy)
       cey = +0.5*(x(1,2)-x(2,1))*
     * (zkxy(1,1)*(y(2,2)-y(1,1))/(2.*VC(1,1))-
     *  zkyy(1,1)*(x(2,2)-x(1,1))/(2.*VC(1,1)))

       ce(1,1) = cex + cey

c     Coefficient cne

c     group related to the approximation of term  
c     nx*(kxx du/dx + kxy du/dy)
       cnex = -0.5*(y(1,2)-y(2,1))*
     * (zkxx(1,1)*(y(1,2)-y(2,1))/(2.*VC(1,1))-
     *  zkxy(1,1)*(x(1,2)-x(2,1))/(2.*VC(1,1)))
      
c     group related to the approximation of term  
c     ny*(kxy du/dx + kyy du/dy)
       cney = +0.5*(x(1,2)-x(2,1))*
     * (zkxy(1,1)*(y(1,2)-y(2,1))/(2.*VC(1,1))-
     *  zkyy(1,1)*(x(1,2)-x(2,1))/(2.*VC(1,1)))

       cne(1,1) = cnex + cney

c     Coefficient cn

c     group related to the approximation of term  
c     nx*(kxx du/dx + kxy du/dy)
       cnx = +0.5*(y(1,2)-y(2,1))*
     * (zkxx(1,1)*(y(2,2)-y(1,1))/(2.*VC(1,1))-
     *  zkxy(1,1)*(x(2,2)-x(1,1))/(2.*VC(1,1)))
      
c     group related to the approximation of term  
c     ny*(kxy du/dx + kyy du/dy)
       cny = -0.5*(x(1,2)-x(2,1))*
     * (zkxy(1,1)*(y(2,2)-y(1,1))/(2.*VC(1,1))-
     *  zkyy(1,1)*(x(2,2)-x(1,1))/(2.*VC(1,1)))

       cn(1,1) = cnx + cny

c      Coefficient cc
       cc(1,1) = -(ce(1,1)+cne(1,1)+cn(1,1))+
     *   alphaw(1)*zl11

c      Right-hand side
       phi(1,1) = gammaw(1)*zl11

c       Corner (M,1)

c----------------------  STENCIL --------------------------------
c
c
c
c              NW(M,1)       N(M,1)          
c             *-------------*
c             !             !               
c             !             !               
c             !             !               
c             !             !              
c             !             !               
c             !             !               
c             !             !              
c             !             !               
c             !             !               
c             !             !               
c             !W(M,1)       !C(M,1)         
c             *-------------*

c            Coefficient cn
c     group related to the approximation of term  
c     nx*(kxx du/dx + kxy du/dy)

       cnx = +0.5*(y(M,2)-y(M-1,1))*
     *  (zkxx(M-1,1)*(y(M-1,2)-y(M,1))/(2.*VC(M-1,1))-
     *   zkxy(M-1,1)*(x(M-1,2)-x(M,1))/(2.*VC(M-1,1)))

c     group related to the approximation of term  
c     ny*(kxy du/dx + kyy du/dy)
       cny = -0.5*(x(M,2)-x(M-1,1))*
     *  (zkxy(M-1,1)*(y(M-1,2)-y(M,1))/(2.*VC(M-1,1))-
     *   zkyy(M-1,1)*(x(M-1,2)-x(M,1))/(2.*VC(M-1,1)))

      cn(M,1) = cnx + cny

c            Coefficient cnw
c     group related to the approximation of term  
c     nx*(kxx du/dx + kxy du/dy)

       cnwx = -0.5*(y(M,2)-y(M-1,1))*
     *  (zkxx(M-1,1)*(y(M,2)-y(M-1,1))/(2.*VC(M-1,1))-
     *   zkxy(M-1,1)*(x(M,2)-x(M-1,1))/(2.*VC(M-1,1)))

c     group related to the approximation of term  
c     ny*(kxy du/dx + kyy du/dy)
       cnwy = +0.5*(x(M,2)-x(M-1,1))*
     *  (zkxy(M-1,1)*(y(M,2)-y(M-1,1))/(2.*VC(M-1,1))-
     *   zkyy(M-1,1)*(x(M,2)-x(M-1,1))/(2.*VC(M-1,1)))

      cnw(M,1) = cnwx + cnwy

c            Coefficient cw
c     group related to the approximation of term  
c     nx*(kxx du/dx + kxy du/dy)

       cwx = -0.5*(y(M,2)-y(M-1,1))*
     *  (zkxx(M-1,1)*(y(M-1,2)-y(M,1))/(2.*VC(M-1,1))-
     *   zkxy(M-1,1)*(x(M-1,2)-x(M,1))/(2.*VC(M-1,1)))

c     group related to the approximation of term  
c     ny*(kxy du/dx + kyy du/dy)
       cwy = +0.5*(x(M,2)-x(M-1,1))*
     *  (zkxy(M-1,1)*(y(M-1,2)-y(M,1))/(2.*VC(M-1,1))-
     *   zkyy(M-1,1)*(x(M-1,2)-x(M,1))/(2.*VC(M-1,1)))

      cw(M,1) = cwx + cwy

c     Coefficient cc
      cc(M,1) = -(cn(M,1)+cnw(M,1)+cw(M,1))+
     * alphae(1)*zlM1

c     Right-hand side 
      phi(M,1) = gammae(1)*zlM1

c       Corner (M,N)

c----------------------  STENCIL --------------------------------
c
c
c
c              W(M,N)       C(M,N)          
c             *-------------*
c             !             !               
c             !             !               
c             !             !               
c             !             !              
c             !             !               
c             !             !               
c             !             !              
c             !             !               
c             !             !               
c             !SW(M,N)      !S(M,N)
c             *-------------* 
c  
c      Coefficient cw

c     group related to the approximation of term  
c     nx*(kxx du/dx + kxy du/dy)

      cwx = -0.5*(y(M-1,N)-y(M,N-1))*
     * (zkxx(M-1,N-1)*(y(M,N)-y(M-1,N-1))
     */(2.*VC(M-1,N-1))- 
     *  zkxy(M-1,N-1)*(x(M,N)-x(M-1,N-1))
     */(2.*VC(M-1,N-1))) 


c     group related to the approximation of term  
c     ny*(kxy du/dx + kyy du/dy)

      cwy = +0.5*(x(M-1,N)-x(M,N-1))*
     * (zkxy(M-1,N-1)*(y(M,N)-y(M-1,N-1))
     * /(2.*VC(M-1,N-1))- 
     *  zkyy(M-1,N-1)*(x(M,N)-x(M-1,N-1))
     * /(2.*VC(M-1,N-1)))

      cw(M,N) = cwx + cwy 

c      Coefficient csw

c     group related to the approximation of term  
c     nx*(kxx du/dx + kxy du/dy)

      cswx = -0.5*(y(M-1,N)-y(M,N-1))*
     * (zkxx(M-1,N-1)*(y(M-1,N)-y(M,N-1)) 
     * /(2.*VC(M-1,N-1))- 
     *  zkxy(M-1,N-1)*(x(M-1,N)-x(M,N-1)) 
     * /(2.*VC(M-1,N-1))) 


c     group related to the approximation of term  
c     ny*(kxy du/dx + kyy du/dy)

      cswy = +0.5*(x(M-1,N)-x(M,N-1))*
     * (zkxy(M-1,N-1)*(y(M-1,N)-y(M,N-1)) 
     * /(2.*VC(M-1,N-1))- 
     *  zkyy(M-1,N-1)*(x(M-1,N)-x(M,N-1))
     * /(2.*VC(M-1,N-1))) 

      csw(M,N) = cswx + cswy 
c      Coefficient cs

c     group related to the approximation of term  
c     nx*(kxx du/dx + kxy du/dy)

      csx = +0.5*(y(M-1,N)-y(M,N-1))*
     * (zkxx(M-1,N-1)*(y(M,N)-y(M-1,N-1)) 
     * /(2.*VC(M-1,N-1))- 
     *  zkxy(M-1,N-1)*(x(M,N)-x(M-1,N-1)) 
     * /(2.*VC(M-1,N-1))) 


c     group related to the approximation of term  
c     ny*(kxy du/dx + kyy du/dy)

      csy = -0.5*(x(M-1,N)-x(M,N-1))*
     * (zkxy(M-1,N-1)*(y(M,N)-y(M-1,N-1)) 
     * /(2.*VC(M-1,N-1))- 
     *  zkyy(M-1,N-1)*(x(M,N)-x(M-1,N-1))
     * /(2.*VC(M-1,N-1))) 

      cs(M,N) = csx + csy 

c     Coefficient cc

      cc(M,N) = -(cw(M,N)+csw(M,N)+cs(M,N))+
     * alphae(N)*zlMN

c     Right-hand side
      phi(M,N) = gammae(N)*zlMN

c        Corner (1,N)

c----------------------  STENCIL --------------------------------
c
c
c
c              C(1,N)       E(1,N)          
c             *-------------*----
c             !             !               
c             !             !               
c             !             !               
c             !             !              
c             !             !               
c             !             !               
c             !             !              
c             !             !               
c             !             !               
c             !S(1,N)       !SE(1,N)
c             *-------------* 
c             !
c             !

c      Coefficient cs

c     group related to the approximation of term  
c     nx*(kxx du/dx + kxy du/dy)
      
      csx = +0.5*(y(2,N)-y(1,N-1))*
     * (zkxx(1,N-1)*(y(1,N)-y(2,N-1))/(2.*VC(1,N-1))-
     *  zkxy(1,N-1)*(x(1,N)-x(2,N-1))/(2.*VC(1,N-1)))
     


c     group related to the approximation of term  
c     ny*(kxy du/dx + kyy du/dy)

      csy = -0.5*(x(2,N)-x(1,N-1))*
     * (zkxy(1,N-1)*(y(1,N)-y(2,N-1))/(2.*VC(1,N-1))-
     *  zkyy(1,N-1)*(x(1,N)-x(2,N-1))/(2.*VC(1,N-1)))

        cs(1,N) = csx + csy

c      Coefficient cse

c     group related to the approximation of term  
c     nx*(kxx du/dx + kxy du/dy)
      
      csex = -0.5*(y(2,N)-y(1,N-1))*
     * (zkxx(1,N-1)*(y(2,N)-y(1,N-1))/(2.*VC(1,N-1))-
     *  zkxy(1,N-1)*(x(2,N)-x(1,N-1))/(2.*VC(1,N-1)))
     


c     group related to the approximation of term  
c     ny*(kxy du/dx + kyy du/dy)

      csey = +0.5*(x(2,N)-x(1,N-1))*
     * (zkxy(1,N-1)*(y(2,N)-y(1,N-1))/(2.*VC(1,N-1))-
     *  zkyy(1,N-1)*(x(2,N)-x(1,N-1))/(2.*VC(1,N-1)))

        cse(1,N) = csex + csey

c      Coefficient ce

c     group related to the approximation of term  
c     nx*(kxx du/dx + kxy du/dy)
      
      cex = -0.5*(y(2,N)-y(1,N-1))*
     * (zkxx(1,N-1)*(y(1,N)-y(2,N-1))/(2.*VC(1,N-1))-
     *  zkxy(1,N-1)*(x(1,N)-x(2,N-1))/(2.*VC(1,N-1)))
     


c     group related to the approximation of term  
c     ny*(kxy du/dx + kyy du/dy)

      cey = +0.5*(x(2,N)-x(1,N-1))*
     * (zkxy(1,N-1)*(y(1,N)-y(2,N-1))/(2.*VC(1,N-1))-
     *  zkyy(1,N-1)*(x(1,N)-x(2,N-1))/(2.*VC(1,N-1)))

        ce(1,N) = cex + cey

c      Coefficient cc 
       
        cc(1,N) = -(cs(1,N)+cse(1,N)+ce(1,N))+
     *   alphaw(N)*zl1N

c      Right-hand side
        phi(1,N) = gammaw(N)*zl1N

         return
         end
c**************************************************************

      subroutine siedel
c
c     SOLUTION for the system of linear algebraic equations
c     by the GAUSS-SIEDEL iteration method
c
      implicit real*8 (a-h,o-z)

c     M,N - number of nodes in the grid 
      common/MN/ M,N

c     u - array (1 to M and 1 to N), solution for the finite-difference scheme
      common/U/ U(200,200)
c    
c     X,Y - arrays (1 to M and 1 to N), coordinates of grid nodes
      common/xy/ x(200,200),y(200,200)
c     =================================================================
c     CC,CE,CNE,CN,CNW,CW,CSW,CS,CSE, PHI - arrays for coefficients
c                                           and the right-hand side
c     ( 2 to M-1 and 2 to N-1 ) in the finite-difference equations
c
c     E(i,j)*U(i+1,j) + NE(i,j)*U(i+1,j+1) + N(i,j)*U(i,j+1) +
c
c     NW(i,j)*U(i-1,j+1) + W(i,j)*U(i-1,j) +SW(i,j)*U(i-1,j-1) +
c
c     S(i,j)*U(i,j-1) + SE(i,j)*U(i+1,j-1) = PHI(i,j)
c


c----------------------  STENCIL --------------------------------
c
c
c
c              NW(i,j)       N(i,j)          NE(i,j)
c             *-------------*---------------*
c             !             !               !
c             !             !               !
c             !             !               !
c             !             !               !
c             !             !               !
c             !             !               !
c             !             !               !
c             !             !               !
c             !             !               !
c             !             !               !
c             !W(i,j)       !C(i,j)         !E(i,j)
c             *-------------*---------------*
c             !             !               !
c             !             !               !
c             !             !               !
c             !             !               !
c             !             !               !
c             !             !               !
c             !             !               !
c             !             !               !
c             !             !               !
c             !             !               !
c             !SW(i,j)      !S(i,j)         !SE(i,j)
c             *-------------*---------------*
c
c
c
c
c
c----------------------  STENCIL --------------------------------

c
c
      common/coefds/ cc(200,200),ce(200,200),cne(200,200),
     *               cn(200,200),cnw(200,200),cw(200,200),
     *               csw(200,200),cs(200,200),
     *               cse(200,200),phi(200,200)
c=================================================================

c     eps - constant in the condition of finishing the iteration in 
c     the Gauss-Siedel iteration procedure.  The termination condition is
c
c             (S+1)     (S)
c     max( abs( U(i,j) - U(i,j))) < eps
c     i,j
c 
c     where S is the iteration number.
c

      common/eps/ eps
c=====================================================================
c     SET an INITIAL value for the iteration solution

      do  i=1, M
      do  j=1, N
      U(i,j)=0.d0
      end do
      end do

c     Beginning of the iteration process
c

5      continue

       EPSMAX= 0.D0

c      Internal Nodes i=2,...,M-1 ; J=2,...,N-1

       do  i=2, M-1
       do  j=2, N-1
       USP1=( phi(i,j)-
     * (CE(i,j)*U(i+1,j) + CNE(i,j)*U(i+1,j+1) + CN(i,j)*U(i,j+1) +
     * CNW(i,j)*U(i-1,j+1) + CW(i,j)*U(i-1,j) +CSW(i,j)*U(i-1,j-1) +
     * CS(i,j)*U(i,j-1) + CSE(i,j)*U(i+1,j-1)))/CC(i,j)
       EPSMAX=amax1(EPSMAX,dabs(USP1 - U(i,j)))
       U(i,j)=USP1
       end do
       end do

c      South-boundary i=2,...,M-1 ; j=1
       j=1
       do  i=2, M-1
       USP1=( phi(i,j)-
     * (CE(i,j)*U(i+1,j) + CNE(i,j)*U(i+1,j+1) + CN(i,j)*U(i,j+1) +
     * CNW(i,j)*U(i-1,j+1) + CW(i,j)*U(i-1,j)))/CC(i,j)
       EPSMAX=amax1(EPSMAX,dabs(USP1 - U(i,j)))
       U(i,j)=USP1
       end do

c      North boundary i=2,...,M-1 ; j=N
        j=N
       do  i=2, M-1
       USP1=( phi(i,j)-(CE(i,j)*U(i+1,j)  +
     *  CW(i,j)*U(i-1,j) +CSW(i,j)*U(i-1,j-1) +
     * CS(i,j)*U(i,j-1) + CSE(i,j)*U(i+1,j-1)))/CC(i,j)
       EPSMAX=amax1(EPSMAX,dabs(USP1 - U(i,j)))
       U(i,j)=USP1
       end do

c      East boundary i=M, j=2,...,N-1
       i=M
       do  j=2, N-1
       USP1=( phi(i,j)-( CN(i,j)*U(i,j+1) +
     * CNW(i,j)*U(i-1,j+1) + CW(i,j)*U(i-1,j) +CSW(i,j)*U(i-1,j-1) +
     * CS(i,j)*U(i,j-1)))/CC(i,j)
       EPSMAX=amax1(EPSMAX,dabs(USP1 - U(i,j)))
       U(i,j)=USP1
       end do




c      West boundary i=1 ; j=2,...,N-1
       i=1
       do  j=2, N-1
       USP1=( phi(i,j)-(CE(i,j)*U(i+1,j) +
     * CNE(i,j)*U(i+1,j+1) + CN(i,j)*U(i,j+1)+
     * CS(i,j)*U(i,j-1) + CSE(i,j)*U(i+1,j-1)))/CC(i,j)
       EPSMAX=amax1(EPSMAX,dabs(USP1 - U(i,j)))
       U(i,j)=USP1
       end do

c      Corner (1,1)
c
       i=1
       j=1
       USP1=( phi(i,j)-
     * (CE(i,j)*U(i+1,j)+CNE(i,j)*U(i+1,j+1)+CN(i,j)*U(i,j+1)
     * ))/CC(i,j)
       EPSMAX=amax1(EPSMAX,dabs(USP1 - U(i,j)))
       U(i,j)=USP1

c      Corner (M,1)
        i=M
        j=1
       USP1=( phi(i,j)-( CN(i,j)*U(i,j+1) +
     * CNW(i,j)*U(i-1,j+1) + CW(i,j)*U(i-1,j)))/CC(i,j)
       EPSMAX=amax1(EPSMAX,dabs(USP1 - U(i,j)))
       U(i,j)=USP1

c      Corner (M,N)
       i=M
       j=N
       USP1=( phi(i,j)-(CW(i,j)*U(i-1,j)
     * +CSW(i,j)*U(i-1,j-1) +
     * CS(i,j)*U(i,j-1) ))/CC(i,j)
       EPSMAX=amax1(EPSMAX,dabs(USP1 - U(i,j)))
       U(i,j)=USP1

c      Corner (1,N)
       i=1
       j=N
       USP1=(phi(i,j)-(CE(i,j)*U(i+1,j)  +
     * CS(i,j)*U(i,j-1) + CSE(i,j)*U(i+1,j-1)))/CC(i,j)
       EPSMAX=amax1(EPSMAX,dabs(USP1 - U(i,j)))
       U(i,j)=USP1

c     checking the termination condition
       if(EPSMAX.GT.EPS) goto 5

       return
       end
c*******************************************************************
      
       subroutine mkgrid
c      This is a subroutine for generation of a smooth grid in 2D
c      as it is shown in the book.
c
c      Left boundary x=0,
c      Right boundary x=1,
c      Bottom boundary y=0,
c      Upper boundary y(x)=1.+0.5*sin(2.*pi*X)
c           _
c          / \
c         /   \
c        /     \
c       /       \
c      /         \
c      I          \                   /I
c      I           \                 / I
c      I            \               /  I
c      I             \             /   I
c      I              \           /    I
c      I               \         /     I
c      I                \       /      I
c      I                 \     /       I
c      I                  \   /        I
c      I                   \_/         I
c      I                               I
c      I_______________________________I
c
c
c      
c 

      implicit real*8 (a-h,o-z)

c     M,N - number of nodes in the grid 
      common/MN/ M,N

c     X,Y - arrays (1 to M and 1 to N), coordinates of grid nodes
      common/xy/ x(200,200),y(200,200)

      M=17

      N=17 

      pi=acos(-1.)

      hx=1./(M-1.)


      do i=1, M
      do j=1, N
      xi=(i-1)*hx
      et=(j-1.)/(N-1)
      hyj=(1.+0.5*sin(2.*pi*xi))/(M-1.)
      x(i,j)=xi
      y(i,j)=(j-1.)*hyj 
      end do
      end do

      return
      end
c**********************************************************************

      subroutine exact
c     Compute the exact solution for the test problem
c     (Dirichlet boundary problem for the Poisson equation)
c     in nodes
c     Results in array
c     uex - 1 to M and 1 to N

      implicit real*8 (a-h,o-z)

c     M,N - number of nodes in the grid 
      common/MN/ M,N

c     X,Y - arrays (1 to M and 1 to N), coordinates of grid nodes
      common/xy/ x(200,200),y(200,200)

c     UEX - array 1 to M and 1 to N; Exact solution
      common/uex/ uex(200,200)

      pi=acos(-1.)

      do i=1,M
      do j=1,N
      uex(i,j)=sin(pi*x(i,j))*sin(pi*y(i,j))
      end do
      end do

      return
      end
c*****************************************************************

      subroutine output

c     Output subroutine 
c     prints the approximate solution,
c     exact solution and errors for
c     the test problem

      implicit real*8 (a-h,o-z)

c     M,N - number of nodes in the grid 
      common/MN/ M,N

c     X,Y - arrays (1 to M and 1 to N), coordinates of grid nodes
      common/xy/ x(200,200),y(200,200)

c     UEX - array 1 to M and 1 to N; Exact solution
      common/uex/ uex(200,200)

c     u - array (1 to M and 1 to N), solution for the finite-difference scheme
      common/U/ U(200,200)

c     ERR - array 1 to M and 1 to N; Error between the exact and
c           approximate solutions in nodes
      common/err/ err(200,200)

c     Max-norm and L2-norm of the error between the exact and approximate
c     solutions in nodes
      common/errors/supn,zL2N


c     Open the file p2drnc.output
c     for test output of program p2drnc
      open(unit=1,file='p2drnc.output')
      
      write(1,33) 
33    format(15X,'TEST OUTPUT FOR PROGRAM p2drnc')
      write(1,*)
      
      write(1,8) M,N, supn, zL2N
8     format(2X,'M=',I5,2X,'N=',I5,2X,
     *       'MAX-NORM = ',E12.5,2X,'  L2-NORM = ',E12.5)

c     Print of the approximate and exact solutions of the test problem

c     Print for I=I1,I2,I3 and for all J=1,...,N

      I1=2
      I2=9
      I3=16
      write(1,9) I1
9     format(10X,'Approximate and Exact Solutions, and Error for',
     * ' I=',I5)
       write(1,*)

      write(1,10)
10    format(5X,'j',7X,' X',12X,' Y',10X,'Appr.Sol.',
     * 4X,'Exact Sol.',5X,'Error')
       i=I1
      do j=1,N
      write(1,11) j,x(i,j),y(i,j),u(i,j),uex(i,j),err(i,j)
      end do
       write(1,*)
       write(1,*)
11    format(1X, i5,5(2X,E12.5))

      write(1,9) I2
       write(1,*)
       i=I2
      do j=1,N
      write(1,11) j,x(i,j),y(i,j),u(i,j),uex(i,j),err(i,j)
      end do
       write(1,*)
       write(1,*)

      write(1,9) I3
       write(1,*)
       i=I3
      do j=1,N
      write(1,11) j,x(i,j),y(i,j),u(i,j),uex(i,j),err(i,j)
      end do
       write(1,*)
       write(1,*)

      return
      end
c*********************************************************************

      subroutine error
c     compute the Max and L2 norm of
c     error for the test problem
      implicit real*8 (a-h,o-z)

c     M,N - number of nodes in the grid 
      common/MN/ M,N

c     X,Y - arrays (1 to M and 1 to N), coordinates of grid nodes
      common/xy/ x(200,200),y(200,200)

c     UEX - array 1 to M and 1 to N; Exact solution
      common/uex/ uex(200,200)

c     u - array (1 to M and 1 to N), solution for the finite-difference scheme
      common/U/ U(200,200)

c     ERR - array 1 to M and 1 to N; Error between the exact and
c           approximate solutions in nodes
      common/err/ err(200,200)

c     Max-norm and L2-norm of the error between the exact and approximate
c     solutions in nodes
      common/errors/supn,zL2N

c     VN - array for the volume of nodes
      dimension VN(200,200),VC(200,200)
c     Comments - this volume is needed in this subroutine
c     only for computing the L2 norm of error	

c     ZLS, ZLE, ZLN, ZLW - array for the length of nodes
c                          on the boundaries
c     ZLS - south, ZLE - east, ZLN - north, ZLW - west.
      dimension zls(200), zle(200), zln(200), zlw(200)
c     Comments - this length is needed in this subroutine
c     only for computing the L2 norm of error	

c     Computing length of nodes zls,zle,zln,zlw

c    South and North boundary - internal nodes
      do i=2,M-1
      zls(i) = 0.5*sqrt((x(i+1,1)-x(i-1,1))**2 +
     *              (y(i+1,1)-y(i-1,1))**2)

      zln(i) = 0.5*sqrt((x(i+1,N)-x(i-1,N))**2 +
     *              (y(i+1,N)-y(i-1,N))**2)
      end do

c     East and West boundary - internal nodes
      do j=2,N-1
      zle(j) = 0.5*sqrt((x(M,j+1)-x(M,j-1))**2 +
     *              (y(M,j+1)-y(M,j-1))**2)

      zlw(j) = 0.5*sqrt((x(1,j+1)-x(1,j-1))**2 +
     *              (y(1,j+1)-y(1,j-1))**2)
      end do

c     Corner (1,1) 
      zl11 = 0.5*sqrt((x(1,2)-x(2,1))**2 +
     *            (y(1,2)-y(2,1))**2)

c     Corner (M,1)
      zlM1 = 0.5*sqrt((x(M-1,1)-x(M,2))**2 + 
     *            (y(M-1,1)-y(M,2))**2)

c     Corner (M,N)
      zlMN = 0.5*sqrt((x(M,N-1)-x(M-1,N))**2 +
     *            (y(M,N-1)-y(M-1,N))**2)

c     Corner (1,N)
      zl1N = 0.5*sqrt((x(2,N)-x(1,N-1))**2 +
     *            (y(2,N)-y(1,N-1))**2) 
c     ================================================================	 

c     Computing the exact solution
      call exact
c     
c     computing the volume of the cell VC(i,j)

      do  i=1,M-1
      do  j=1,N-1

      VC(i,j) =0.5*((x(i,j)-x(i+1,j+1))*(y(i+1,j)-y(i,j+1)) -
     *              (x(i+1,j)-x(i,j+1))*(y(i,j)-y(i+1,j+1)))

      end do
      end do

c     Computing the volume of the nodes

      do i=2,M-1
      do j=2,N-1
c     Two different choices for VN(i,j)
c     First - average for the volumes of adjacent cells
c     VN(i,j)=0.25*(VC(i,j) + VC(i-1,j) + VC(i-1,j-1) + VC(i,j-1))
c     Second "DIAMOND" volume
      VN(i,j) =0.25*((x(i-1,j)-x(i+1,j))*(y(i,j-1)-y(i,j+1)) -
     *              (x(i,j-1)-x(i,j+1))*(y(i-1,j)-y(i+1,j)))
 
      end do
      end do

c     Error
c     -----

      do i=1,M
      do j=1,N
      err(i,j)=uex(i,j)-u(i,j)
      end do
      end do
     

c     Max-Norm
c     --------
      supn=0.d0
      do i=1,M
      do j=1,N
      supn=amax1(supn,abs(err(i,j)))
      end do
      end do
      
c     L  - norm
c      2
c     ----------

      zL2N=0.0d0
      do i=2,M-1
      do j=2,N-1
      zL2N = zL2N + err(i,j)**2*VN(i,j)
      end do
      end do
c     
c     Boundary terms
c
      i=1
      do j=2,N-1
      zL2N = zL2N + err(i,j)**2*zlw(j)
      end do

      i=M
      do j=2,N-1
      zL2N = zL2N + err(i,j)**2*zle(j)
      end do
      
      j=1
      do i=2,M-1
      zL2N = zL2N + err(i,j)**2*zls(i) 
      end do

      j=N
      do i=2,M-1
      zL2N = zL2N + err(i,j)**2*zln(i) 
      end do
c
c     Corners
c        
       i=1
       j=1
      zL2N = zL2N + err(i,j)**2*zl11 

       i=1
       j=N  
      zL2N = zL2N + err(i,j)**2*zl1N  

       i=M
       j=N
      zL2N = zL2N + err(i,j)**2*zlMN  
 
       i=M
       j=1
      zL2N = zL2N + err(i,j)**2*zlM1
c
c
c  
      zL2N=dsqrt(zL2N)
      
      return
      end
c**********************************************************************

      subroutine bcon
c     The subroutine computes the values of arrays
c     related to functions alpha and gamma in
c     Robin boundary conditions
c     in the finite-difference scheme.

c     falphas(x,y),falphae(x,y),falphan(x,y),falphaw(x,y)
c     fgammas(x,y),fgammae(x,y),fgamman(x,y),fgammaw(x,y)
c     functions which compute functions alpha and gamma
c     in Robin boundary conditions on
c     related parts of the boundary  
     
      implicit real*8 (a-h,o-z)
c     M,N - number of nodes in the grid 
      common/MN/ M,N

c     X,Y - arrays (1 to M and 1 to N), coordinates of grid nodes
      common/xy/ x(200,200),y(200,200)
   
c     ALPHAS, ALPHAE, ALPHAN, ALPHAW - arrays which correspond
c     to function alpha(x,y) in the left-hand side of the boundary
c     conditions:    
c     ALPHAS, ALPHAN - south and north boundary - arrays (1 to M )
c     ALPHAW, ALPHAE - west and east boundary - arrays ( 1 to N )
      common/alpha/ alphas(200), alphae(200),
     * alphan(200), alphaw(200)

c     GAMMAS, GAMMAE, GAMMAN, GAMMAW - arrays which correspond
c     to function gamma(x,y) in the right-hand side of the
c     boundary conditions:
c     GAMMAS, GAMMAN - south and north boundary - arrays (1 to M )
c     GAMMAW, GAMMAE - west and east boundary - arrays ( 1 to N )
      common/alpha/ gammas(200), gammae(200),
     * gamman(200), gammaw(200)

c     Set Robin boundary conditions
c     -----------------------------------

c     North and South Boundary
      do  i=1,M
      alphan(i) = falphan(x(i,N),y(i,N))
      alphas(i) = falphas(x(i,1),y(i,1))
      gamman(i) = fgamman(x(i,N),y(i,N))
      gammas(i) = fgammas(x(i,1),y(i,1))
      end do

c     East and West Boundary
      do  j=1,N
      alphae(j) = falphae(x(M,j),y(M,j))
      alphaw(j) = falphaw(x(1,j),y(1,j))
      gammae(j) = fgammae(x(M,j),y(M,j))
      gammaw(j) = fgammaw(x(1,j),y(1,j))
      end do

      return
      end
c***********************************************************************

      real function falphan*8(x,y)
c     function falphan - returned values of function alpha
c     in the left-hand side of Robin boundary conditions of
c     a differential problem
c     North - Top - boundary -> j=N
      implicit real*8 (a-h,o-z)
      falphan=1.                  
      return
      end
c***********************************************************************
      real function falphas*8(x,y)
c     function falphas - returned values of function alpha
c     in the left-hand side of Robin boundary conditions of
c     a differential problem
c     South - Bottom - boundary -> j=1
      implicit real*8 (a-h,o-z)
      falphas=1.                  
      return
      end
c***********************************************************************
      real function falphae*8(x,y)
c     function falphae - returned values of function alpha
c     in the left-hand side of Robin boundary conditions of
c     a differential problem
c     East - Right - boundary -> i=M
      implicit real*8 (a-h,o-z)
      falphae=1.                  
      return
      end
c***********************************************************************
      real function falphaw*8(x,y)
c     function falphaw - returned values of function alpha
c     in the left-hand side of Robin boundary conditions of
c     a differential problem
c     West - Left - boundary -> i=1
      implicit real*8 (a-h,o-z)
      falphaw=1.                  
      return
      end
c***********************************************************************
      real function fgamman*8(x,y)
c     function fgamman - returned values of function gamma
c     in the right-hand side of Robin boundary conditions of
c     a differential problem
c     North - Top - boundary -> j=N
      implicit real*8 (a-h,o-z)
      pi=acos(-1.)
     
            d1  = 1.0d0 + 2.0d0*x**2 + y**2
            d2  = 1.0d0 + x**2 + 2.0d0*y**2
            d1x = 4.0d0*x
            d1y = 2.0d0*y
            d2x = 2.0d0*x
            d2y = 4.0d0*y
            t  = 5.d0*pi/12.d0
            ct = dcos(t)
            st = dsin(t)
            zkxx = d1*ct*ct + d2*st*st
            zkxy = (d1-d2)*ct*st
            zkyy = d1*st*st + d2*ct*ct
c
            sx  = sin(pi*x)
            cx  = cos(pi*x)
            sy  = sin(pi*y)
            cy  = cos(pi*y)
            fx       =     pi*cx*sy
            fy       =     pi*sx*cy
             fe=sx*sy
           
      znx=-pi*cos(2.*pi*x)/sqrt(1.+pi**2*cos(2.*pi*x)**2)
      zny=+1./sqrt(1.+pi**2*cos(2.*pi*x)**2)
c
c     Components of vector K*GRAD f

      zkgx=zkxx*fx+zkxy*fy
      zkgy=zkxy*fx+zkyy*fy

      fgamman=znx*zkgx+zny*zkgy + fe

         
c     fgamman=+(beta*fx+gamma*fy) + fe        
      return
      end
c***********************************************************************
      real function fgammas*8(x,y)
c     function fgammas - returned values of function gamma
c     in the right-hand side of Robin boundary conditions of
c     a differential problem
c     South - Bottom - boundary -> j=1
      implicit real*8 (a-h,o-z)
      pi=acos(-1.)
     
            d1  = 1.0d0 + 2.0d0*x**2 + y**2
            d2  = 1.0d0 + x**2 + 2.0d0*y**2
            d1x = 4.0d0*x
            d1y = 2.0d0*y
            d2x = 2.0d0*x
            d2y = 4.0d0*y
            t  = 5.d0*pi/12.d0
            ct = dcos(t)
            st = dsin(t)
            zkxx = d1*ct*ct + d2*st*st
            zkxy = (d1-d2)*ct*st
            zkyy = d1*st*st + d2*ct*ct
c
            sx  = sin(pi*x)
            cx  = cos(pi*x)
            sy  = sin(pi*y)
            cy  = cos(pi*y)
            fx       =     pi*cx*sy
            fy       =     pi*sx*cy
            fe=sx*sy

        

      fgammas=-(zkxy*fx+zkyy*fy) + fe      
      return
      end
c***********************************************************************
      real function fgammae*8(x,y)
c     function fgammae - returned values of function gamma
c     in the right-hand side of Robin boundary conditions of
c     a differential problem
c     East - Right - boundary -> i=M
      implicit real*8 (a-h,o-z)
      pi=acos(-1.)
     
            d1  = 1.0d0 + 2.0d0*x**2 + y**2
            d2  = 1.0d0 + x**2 + 2.0d0*y**2
            d1x = 4.0d0*x
            d1y = 2.0d0*y
            d2x = 2.0d0*x
            d2y = 4.0d0*y
            t  = 5.d0*pi/12.d0
            ct = dcos(t)
            st = dsin(t)
            zkxx = d1*ct*ct + d2*st*st
            zkxy = (d1-d2)*ct*st
            zkyy = d1*st*st + d2*ct*ct
c
            sx  = sin(pi*x)
            cx  = cos(pi*x)
            sy  = sin(pi*y)
            cy  = cos(pi*y)
            fx       =     pi*cx*sy
            fy       =     pi*sx*cy
c      
            fe=sx*sy



      fgammae=+(zkxx*fx+zkxy*fy) + fe     
      return
      end
c***********************************************************************
      real function fgammaw*8(x,y)
c     function fgammaw - returned values of function gamma
c     in the right-hand side of Robin boundary conditions of
c     a differential problem
c     West - Left - boundary -> i=1
      implicit real*8 (a-h,o-z)
      pi=acos(-1.)
     
            d1  = 1.0d0 + 2.0d0*x**2 + y**2
            d2  = 1.0d0 + x**2 + 2.0d0*y**2
            d1x = 4.0d0*x
            d1y = 2.0d0*y
            d2x = 2.0d0*x
            d2y = 4.0d0*y
            t  = 5.d0*pi/12.d0
            ct = dcos(t)
            st = dsin(t)
            zkxx = d1*ct*ct + d2*st*st
            zkxy = (d1-d2)*ct*st
            zkyy = d1*st*st + d2*ct*ct
c
            sx  = sin(pi*x)
            cx  = cos(pi*x)
            sy  = sin(pi*y)
            cy  = cos(pi*y)
            fx       =     pi*cx*sy
            fy       =     pi*sx*cy
c      
            fe=sx*sy


      fgammaw=-(zkxx*fx+zkxy*fy) + fe     
      return
      end
c***********************************************************************
      subroutine coeff
c     Computation of coefficients K.. of the finite-difference
c     scheme in cells

      implicit real*8 (a-h,o-z)
c     M,N - number of nodes in the grid 
      common/MN/ M,N

c     X,Y - arrays (1 to M and 1 to N), coordinates of grid nodes
      common/xy/ x(200,200),y(200,200)

c     ZK.. - arrays of (1 to M-1 and 1 to N-1), value of the matrix of
c     coefficient K in cells
      common/k/ zkxx(200,200), zkxy(200,200),zkyy(200,200)


      do i=1,M-1
      do j=1,N-1

c     coordinates of the center of cell I,J

      xcij=0.25*(x(i,j)+x(i+1,j)+x(i+1,j+1)+x(i,j+1))
      ycij=0.25*(y(i,j)+y(i+1,j)+y(i+1,j+1)+y(i,j+1))

c     computation of coefficients

      zkxx(i,j)=fkxx(xcij,ycij)                                         
      zkxy(i,j)=fkxy(xcij,ycij)
      zkyy(i,j)=fkyy(xcij,ycij)
      
      end do
      end do

      return
      end
c*********************************************************************

      real function fkxx*8 (x,y)
c     function fkxx(x,y) computes the coefficients
c     KXX of the original differential equation in point (x,y)
      implicit real*8 (a-h,o-z)
      pi=acos(-1.)
      d1 = 1.0d0 + 2.0d0*x**2 + y**2
      d2 = 1.0d0 + x**2 + 2.0d0*y**2
      t = 5.d0*pi/12.d0
      ct=cos(t)
      st=sin(t)
      fkxx=d1*ct*ct + d2*st*st
      return
      end
c*********************************************************************

      real function fkxy*8 (x,y)
c     function fkxy(x,y) computes the coefficients
c     KXX of the original differential equation in point (x,y)
      implicit real*8 (a-h,o-z)
      pi=acos(-1.)
      d1 = 1.0d0 + 2.0d0*x**2 + y**2
      d2 = 1.0d0 + x**2 + 2.0d0*y**2
      t = 5.d0*pi/12.d0
      ct=cos(t)
      st=sin(t)
      fkxy=(d1-d2)*ct*st
      return
      end
c*********************************************************************

      real function fkyy*8 (x,y)
c     function fkxy(x,y) computes the coefficients
c     KXX of the original differential equation in point (x,y)
      implicit real*8 (a-h,o-z)
      pi=acos(-1.)
      d1 = 1.0d0 + 2.0d0*x**2 + y**2
      d2 = 1.0d0 + x**2 + 2.0d0*y**2
      t = 5.d0*pi/12.d0
      ct=cos(t)
      st=sin(t)
      fkyy=d1*st*st + d2*ct*ct
      return
      end
c*********************************************************************

      subroutine right
c     Computation of the right-hand side of the finite-difference equations
c     in nodes
      implicit real*8 (a-h,o-z)

c     M,N - number of nodes in the grid 
      common/MN/ M,N

c     X,Y - arrays (1 to M and 1 to N), coordinates of grid nodes
      common/xy/ x(200,200),y(200,200)

c     F - array (2 to M-1, and 2 to N-1), values of the right-hand
c     side of the differential equation in nodes
      common/f/ f(200,200)

c     comuting the right-hand side of the finite-difference equations
c     ff(x,y) - function which computes the value of the right-hand side
c     of the original differential problem

      do i=2,M-1
      do j=2,N-1
      f(i,j)=ff(x(i,j),y(i,j))
      end do
      end do

      return
      end
c**********************************************************************

      real function ff*8(x,y)
c     function ff(x,y) - computes the value of the right-hand side
c     of the original differential equation in point (x,y)

      implicit real*8 (a-h,o-z)
      pi=acos(-1.)
     
            d1  = 1.0d0 + 2.0d0*x**2 + y**2
            d2  = 1.0d0 + x**2 + 2.0d0*y**2
            d1x = 4.0d0*x
            d1y = 2.0d0*y
            d2x = 2.0d0*x
            d2y = 4.0d0*y
            t  = 5.d0*pi/12.d0
            ct = dcos(t)
            st = dsin(t)
            zkxx = d1*ct*ct + d2*st*st
            zkxy = (d1-d2)*ct*st
            zkyy = d1*st*st + d2*ct*ct
            zkxx_x     = d1x*ct*ct + d2x*st*st
            zkxy_x     = ( d1x - d2x )*ct*st         
            zkxy_y    = ( d1y - d2y )*ct*st         
            zkyy_y     = d1y*st*st + d2y*ct*ct
c
            sx  = sin(pi*x)
            cx  = cos(pi*x)
            sy  = sin(pi*y)
            cy  = cos(pi*y)
            fx       =     pi*cx*sy
            fy       =     pi*sx*cy
            fxx      = -pi*pi*sx*sy
            fxy      =  pi*pi*cx*cy
            fyy      = -pi*pi*sx*sy

            ff = -( zkxx*fxx + 2*zkxy*fxy + zkyy*fyy
     * + zkxx_x*fx + zkxy_y*fx + zkxy_x*fy + zkyy_y*fy)
      return
      end
c************************************************************************




