         program gid1de
         implicit real*8(a-h,o-z)
c        Main program for 1D explicit schemes
c        for Lagrangian gas dynamics.

c        Number of nodes, time and time step
         common/nm/ n
         common/tdt/ t, dt

c        Number of nodes
         n=101

c        Maximum time
         tmax=0.51

c        Time step
         dt=0.00025


c        Set t and tout,dtout

c        Output with time interval dtout
         dtout=0.5 

c        Time for output tout
         tout=dtout
 

   
c         Computation of the initial values
c         of parameters and some useful
c         arrays as the mass of cells and nodes.
c         subroutine ini

          call ini

c         Computation of parameters on the next step

1         continue
c         New time
          t=t+dt

          call stepe

          if(t.ge.tmax) stop
        
c         Output of the results  

          if(t.ge.tout-dt) then

          call output
          tout=tout+dtout
          end if

          go to 1

          end
c================================================
        subroutine ini
        implicit real*8(a-h,o-z)
c       Computation of the initial data
c       and useful arrays
c        x - coordinate
c        w - velocity
c        rho - density
c        p - pressure
c        e - internal energy
c        vn - volume of nodes
c        vc - volume of cells
c        zmn - mass of nodes
c        zmc - mass of cells

        common/nm/ n

        common/x/ x(0:201)
        common/rho/ rho(0:201)
        common/vareps/ e(0:201)
        common/press/ p(0:201)
        common/wx/ w(0:201)
        common/mass/ zmc(0:201), zmn(0:201)
        common/vol/ vc(0:201), vn(0:201)

        
c       Grid 
        h=1./(n-1.)
        do i=1,n
        x(i)=(i-1.)*h     
        end do
c       
c       Fictitious nodes 0 and n+1
        x(0)=x(1)
        x(n+1)=x(n)

        
c       Initial profile for density and internal energy
c       and pressure
        do i=1,n-1
        xm=0.5*(x(i+1)+x(i))
        rho(i)=frho(xm)
        e(i)=fe(xm)
c       Pressure from the equation of state
        p(i)=fp(rho(i),e(i))
        end do

c       Initial profile for velocity
        do i=2,n-1
        w(i)=fw(x(i))
        end do

c       Boundary conditions for velocity
c       Piston problem 
c       left boundary is the surface of the piston
c       and moving with constant velocity
c       w(1)=1.                     
c       right boundary is a stationary wall
c       w(n)=0.

        w(1)=1.
        w(n)=0.



c       Volume of the cells
        do i=0,n
        vc(i)=x(i+1)-x(i)
        end do


c       Mass of the cells
        do i=0,n
        zmc(i)=rho(i)*vc(i)
        end do

c       Mass of the nodes
        do i=1,n
        zmn(i)=0.5*(zmc(i-1)+zmc(i))
        end do

        return
        end 
c==================================================
         real function fp*8(rho,e)
         implicit real*8(a-h,o-z)
c        Equation of state
c        (Ideal gas gamma=5/3)

        gamma=5./3.

        fp = (gamma-1.)*rho*e

        return
        end    
c ==================================================
         real function fc*8(rho,e)
         implicit real*8(a-h,o-z)
c        Adiabatic speed of sound

         gamma=5./3.
         fc=sqrt((gamma-1.)*gamma*e)

        return
         end
c ==================================================
         real function frho*8(x)
         implicit real*8(a-h,o-z)
c       Initial profile for density
        frho = 1.
        return
        end

c ==================================================

         real function fe*8(x)
         implicit real*8(a-h,o-z)
c       Initial profile for internal energy
        fe = 0.0001     
        return
        end

c ==================================================
         real function fw*8(x)
         implicit real*8(a-h,o-z)
c        Initial profile for velocity
        fw = 0.0      
        return
        end
c ==================================================
         subroutine stepe
c        Explicit computations
c        on the new time step
         implicit real*8(a-h,o-z)
       
c        x - coordinate
c        w - velocity
c        rho - density
c        p - pressure
c        e - internal energy
c        vn - volume of nodes
c        vc - volume of cells
c        zmn - mass of nodes
c        zmc - mass of cells

        common/nm/ n
        common/tdt/ t, dt

        common/x/ x(0:201)
        common/rho/ rho(0:201)
        common/vareps/ e(0:201)
        common/press/ p(0:201)
        common/wx/ w(0:201)
        common/mass/ zmc(0:201), zmn(0:201)
        common/vol/ vc(0:201), vn(0:201)

c       Velocity on n+1 - np1 time level
        dimension wnp1(0:201)

       
c       Artificial viscosity
        dimension q(0:201)
        

c       Artificial viscosity
c       Linear artificial viscosity
c       Coefficient of artificial viscosity
        znu=1.5
        gamma=5./3.
        do i=1,n-1
        if(w(i+1)-w(i).ge.0) then
        q(i)=0.
        else
        zfc=fc(rho(i),e(i))
        q(i)=-znu*rho(i)*zfc*(w(i+1)-w(i))
c       Quadratic viscosity
c       q(i)= znu*rho(i)*(w(i+1)-w(i))**2
        end if
        end do
        
c       New velocities
c       boundary conditions
        wnp1(1) = 1.
        wnp1(n) = 0.
c
c       internal nodes from the equation of motion
c       
        do i=2,n-1
        wnp1(i) = w(i)-dt*(p(i)+q(i)-p(i-1)-q(i-1))/zmn(i)
        end do

        
c       New coordinates

        do i=1,n
        x(i)=x(i)+0.5*dt*(wnp1(i)+w(i))
        end do

c       New internal energy
        do i=1,n-1
        e(i)=e(i)-dt*(p(i)+q(i))*
     *  (0.5*(wnp1(i+1)+w(i+1))-0.5*(wnp1(i)+w(i)))/zmc(i)
        end do

c       New volume 
        do i=1,n-1
        vc(i)=x(i+1)-x(i)
        end do

       
c        New density
         do i=1,n-1
         rho(i)=zmc(i)/vc(i)
         end do

c        New pressure
         do i=1,n-1
         p(i)=fp(rho(i),e(i))
         end do

         do i=1,n
         w(i)=wnp1(i)
         end do


        return
         end
c================================================
         subroutine output
         implicit real*8(a-h,o-z)
c        Output of flow parameters

c        x - coordinate
c        w - velocity
c        rho - density
c        p - pressure
c        e - internal energy
c        vn - volume of nodes
c        vc - volume of cells
c        zmn - mass of nodes
c        zmc - mass of cells

        common/nm/ n
        common/tdt/ t, dt

        common/x/ x(0:201)
        common/rho/ rho(0:201)
        common/vareps/ e(0:201)
        common/press/ p(0:201)
        common/wx/ w(0:201)
        common/mass/ zmc(0:201), zmn(0:201)
        common/vol/ vc(0:201), vn(0:201)

        open(unit=1,file='g1de.output')
        
        write(1,*) '               This is test output for program g1de'
        write(1,*)
        write(1,*) '        n=',n,'    t=',t,'     dt=',dt 
        write(1,*)
        write(1,88)
88      format(2X,'i',10X,'x',13X,'w',12X,'rho',14X,'e',13X,'p')

        do i=1,n
        write(1,77) i,x(i),w(i),rho(i),e(i),p(i)
        end do 
                 
77      format(1X,i3,5(2X,e13.6))

        return
        end
        


